@@ -1,11 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* e-cell-text.c - Text cell renderer
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Authors: Miguel de Icaza <miguel@helixcode.com>
- *          Chris Lahey <clahey@helixcode.com>
+ * Authors: Miguel de Icaza <miguel@ximian.com>
+ *          Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999, 2000 Helix Code, Inc.
+ * (C) 1999, 2000 Ximian, Inc.
  *
  * A lot of code taken from:
  *
@@ -49,6 +49,8 @@
 #include <ctype.h>
 #include <math.h>
 
+#define ECT_CLASS(c) (E_CELL_TEXT_CLASS(GTK_OBJECT((c))->klass))
+
 /* This defines a line of text */
 struct line {
 	char *text;	/* Line's text UTF-8, it is a pointer into the text->text string */
@@ -63,7 +65,6 @@ enum {
 
 	ARG_STRIKEOUT_COLUMN,
 	ARG_BOLD_COLUMN,
-	ARG_TEXT_FILTER,
 	ARG_COLOR_COLUMN,
 };
 
@@ -210,6 +211,33 @@ static GdkColor* e_cell_text_get_color (ECellTextView *cell_view, gchar *color_s
 
 static ECellClass *parent_class;
 
+static char *
+ect_get_text (ECellText *cell, ETableModel *model, int col, int row)
+{
+	if (ECT_CLASS(cell)->get_text)
+		return ECT_CLASS(cell)->get_text (cell, model, col, row);
+	else
+		return NULL;
+}
+
+static void
+ect_free_text (ECellText *cell, char *text)
+{
+	if (ECT_CLASS(cell)->free_text)
+		ECT_CLASS(cell)->free_text (cell, text);
+}
+
+static char *
+ect_real_get_text (ECellText *cell, ETableModel *model, int col, int row)
+{
+	return e_table_model_value_at(model, col, row);
+}
+
+static void
+ect_real_free_text (ECellText *cell, char *text)
+{
+}
+
 static void
 ect_queue_redraw (ECellTextView *text_view, int view_col, int view_row)
 {
@@ -581,41 +609,11 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 						       ypos + y1,
 						       lines->text + sel_end - start_char,
 						       end_char - sel_end);
-#if 0 /* Do Bold in EFont directly */
-				/* Draw 1,0 moved image, to simulate bold font */
-				/* We move that to EFont */
-				if (bold) {
-					e_font_draw_utf8_text (drawable, font, text_view->gc, xpos + x1 + 1, ypos + y1,
-						       lines->text,
-						       sel_start - start_char);
-					e_font_draw_utf8_text (drawable, font, fg_gc,
-						       xpos + x1 + 1 + e_font_utf8_text_width (font, lines->text, sel_start - start_char),
-						       ypos + y1,
-						       g_utf8_offset_to_pointer (lines->text, sel_start - start_char),
-						       sel_end - sel_start);
-					e_font_draw_utf8_text (drawable, font, text_view->gc,
-						       xpos + x1 + 1 + e_font_utf8_text_width (font, lines->text, sel_end - start_char),
-						       ypos + y1,
-						       g_utf8_offset_to_pointer (lines->text, sel_end - start_char),
-						       end_char - sel_end);
-				}
-#endif
 			} else {
 				e_font_draw_utf8_text (drawable, font, style, text_view->gc,
 						       xpos + x1, ypos + y1,
 						       lines->text,
 						       lines->length);
-#if 0
-				if (bold) {
-					e_font_draw_ucs2_text (drawable,
-						       font,
-						       text_view->gc,
-						       xpos + x1 + 1,
-						       ypos + y1,
-						       lines->text,
-						       lines->length);
-				}
-#endif
 			}
 			if (edit->selection_start == edit->selection_end &&
 			    edit->selection_start >= start_char &&
@@ -674,42 +672,12 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 					       ypos + y1,
 					       ect->ellipsis ? ect->ellipsis : "...",
 					       ect->ellipsis ? strlen (ect->ellipsis) : 3);
-#if 0
-				if (bold) {
-					e_font_draw_ucs2_text (drawable,
-						       font,
-						       text_view->gc,
-						       xpos + x1 + 1,
-						       ypos + y1,
-						       lines->text,
-						       lines->ellipsis_length);
-					e_font_draw_utf8_text (drawable,
-						       font,
-						       text_view->gc,
-						       xpos + x1 + 1 +
-						       lines->width - text_view->ellipsis_width[style],
-						       ypos + y1,
-						       ect->ellipsis ? ect->ellipsis : "...",
-						       ect->ellipsis ? strlen (ect->ellipsis) : 3);
-				}
-#endif
 			} else {
 				e_font_draw_utf8_text (drawable, font, style, text_view->gc,
 					       xpos + x1,
 					       ypos + y1,
 					       lines->text,
 					       lines->length);
-#if 0
-				if (bold) {
-					e_font_draw_ucs2_text (drawable,
-						       font,
-						       text_view->gc,
-						       xpos + x1 + 1,
-						       ypos + y1,
-						       lines->text,
-						       lines->length);
-				}
-#endif
 			}
 			if (ect->strikeout_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->strikeout_column, row)) {
 				gdk_draw_rectangle (drawable,
@@ -728,133 +696,6 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 
 	gdk_gc_set_clip_rectangle (text_view->gc, NULL);
 	gdk_gc_set_clip_rectangle (fg_gc, NULL);
-#if 0
-	/* Old ECellText */
-
-	int xoff;
-	gboolean edit_display = FALSE;
-
-	/*
-	 * Figure if this cell is being edited
-	 */
-	if (edit_display){
-		CellEdit *edit = text_view->edit;
-		const char *text = gtk_entry_get_text (edit->entry);
-		GdkWChar *p, *text_wc = g_new (GdkWChar, strlen (text) + 1);
-		int text_wc_len = gdk_mbstowcs (text_wc, text, strlen (text));
-		const int cursor_pos = GTK_EDITABLE (edit->entry)->current_pos;
-		const int left_len = gdk_text_width_wc (text_view->font, text_wc, cursor_pos);
-
-		text_wc [text_wc_len] = 0;
-		/*
-		 * Paint
-		 */
-		gdk_gc_set_foreground (text_view->gc, &w->style->base [GTK_STATE_NORMAL]);
-		gdk_draw_rectangle (drawable, text_view->gc, TRUE,
-				    rect.x, rect.y, rect.width, rect.height);
-		gdk_gc_set_foreground (text_view->gc, &w->style->text [GTK_STATE_NORMAL]);
-
-		{
-			GdkGC *gc = text_view->gc;
-			const int y = y2 - font->descent - ((y2-y1-height)/2);
-			int px, i;
-
-			/*
-			 * Border
-			 */
-			x1 += 2;
-			x2--;
-			
-			px = x1;
-
-			/*
-			 * If the cursor is outside the visible range
-			 *
-			 * FIXME: we really want a better behaviour.
-			 */
-			if ((px + left_len) > x2)
-				px -= left_len - (x2-x1);
-
-			/*
-			 * Draw
-			 */
-			for (i = 0, p = text_wc; *p; p++, i++){
-				gdk_draw_text_wc (
-						  drawable, font, gc, px, y, p, 1);
-
-				if (i == cursor_pos){
-					gdk_draw_line (
-						       drawable, gc,
-						       px, y - font->ascent,
-						       px, y + font->descent - 1);
-				}
-
-				px += gdk_text_width_wc (font, p, 1);
-			}
-
-			if (i == cursor_pos){
-				gdk_draw_line (
-					       drawable, gc,
-					       px, y - font->ascent,
-					       px, y + font->descent - 1);
-			}
-		}
-		g_free (text_wc);
-	} else {
-		/*
-		 * Regular cell
-		 */
-		GdkColor *background, *foreground;
-		int width;
-
-		/*
-		 * Border
-		 */
-		x1++;
-		x2--;
-		
-		/*
-		 * Compute draw mode
-		 */
-		switch (ect->justify){
-		case GTK_JUSTIFY_LEFT:
-			xoff = 1;
-			break;
-			
-		case GTK_JUSTIFY_RIGHT:
-			width = 1 + gdk_text_width (font, str, strlen (str));
-			xoff = (x2 - x1) - width;
-			break;
-			
-		case GTK_JUSTIFY_CENTER:
-			xoff = ((x2 - x1) - gdk_text_width (font, str, strlen (str))) / 2;
-			break;
-		default:
-			xoff = 0;
-			g_warning ("Can not handle GTK_JUSTIFY_FILL");
-			break;
-		}
-		
-		
-		if (selected){
-			background = &w->style->bg [GTK_STATE_SELECTED];
-			foreground = &w->style->text [GTK_STATE_SELECTED];
-		} else {
-			background = &w->style->base [GTK_STATE_NORMAL];
-			foreground = &w->style->text [GTK_STATE_NORMAL];
-		}
-		
-		gdk_gc_set_foreground (text_view->gc, background); 
-		gdk_draw_rectangle (drawable, text_view->gc, TRUE,
-				    rect.x, rect.y, rect.width, rect.height);
-		gdk_gc_set_foreground (text_view->gc, foreground);
-
-		gdk_draw_string (
-				 drawable, font, text_view->gc,
-				 x1 + xoff,
-				 y2 - font->descent - ((y2-y1-height)/2), str);
-	}
-#endif
 }
 
 /*
@@ -869,6 +710,15 @@ ect_edit_select_all (ECellTextView *text_view)
 	text_view->edit->selection_end = strlen (text_view->edit->cell.text);
 }
 
+static gboolean
+key_begins_editing (GdkEventKey *event)
+{
+	if (event->length == 0)
+		return FALSE;
+
+	return TRUE;
+}
+
 /*
  * ECell::event method
  */
@@ -913,8 +763,10 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			return_val = TRUE;
 			break;
 		}
-		
-		if ((!edit_display) && e_table_model_is_cell_editable (ecell_view->e_table_model, model_col, row)) {
+
+		if ((!edit_display) &&
+		    e_table_model_is_cell_editable (ecell_view->e_table_model, model_col, row) &&
+		    key_begins_editing (&event->key)) {
 			  e_table_item_enter_edit (text_view->cell_view.e_table_item_view, view_col, row);
 			  ect_edit_select_all (text_view);
 			  edit = text_view->edit;
@@ -1064,36 +916,6 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 	else
 		return 0;
 	
-#if 0
-	switch (event->type){
-	case GDK_BUTTON_PRESS:
-		/*
-		 * Adjust for the border we use
-		 */
-		event->button.x++;
-		
-		printf ("Button pressed at %g %g\n", event->button.x, event->button.y);
-		if (text_view->edit){
-			printf ("FIXME: Should handle click here\n");
-		} else 
-			e_table_item_enter_edit (text_view->cell_view.e_table_item_view, view_col, row);
-		break;
-
-	case GDK_KEY_PRESS:
-		if (event->key.keyval == GDK_Escape){
-			ect_cancel_edit (text_view);
-			return TRUE;
-		}
-		
-		if (!text_view->edit){
-			e_table_item_enter_edit (text_view->cell_view.e_table_item_view, view_col, row);
-			ect_edit_select_all (text_view);
-		}
-
-		gtk_widget_event (GTK_WIDGET (text_view->edit->entry), event);
-		ect_queue_redraw (text_view, view_col, row);
-		break;
-#endif
 }
 
 /*
@@ -1105,28 +927,16 @@ ect_height (ECellView *ecell_view, int model_col, int view_col, int row)
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
 	EFont *font;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
+	gchar *string;
+	gint value;
 	
 	font = text_view->font;
-	if (ect->filter) {
-		gchar *string;
-		gint value;
 
-		string = (*ect->filter)(e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-		value = e_font_height (font) * number_of_lines(string) + TEXT_PAD;
+	string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	value = e_font_height (font) * number_of_lines(string) + TEXT_PAD;
+	ect_free_text(ect, string);
 
-		g_free(string);
-
-		return value;
-	} else {
-		gchar * string;
-		gint value;
-
-		string = e_table_model_value_at (ecell_view->e_table_model, model_col, row);
-
-		value = e_font_height (font) * number_of_lines (string) + TEXT_PAD;
-
-		return value;
-	}
+	return value;
 }
 
 /*
@@ -1138,6 +948,7 @@ ect_enter_edit (ECellView *ecell_view, int model_col, int view_col, int row)
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
 	CellEdit *edit;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
+	char *temp;
 
 	edit = g_new (CellEdit, 1);
 	text_view->edit = edit;
@@ -1179,11 +990,9 @@ ect_enter_edit (ECellView *ecell_view, int model_col, int view_col, int row)
 	edit->pointer_in = FALSE;
 	edit->default_cursor_shown = TRUE;
 	
-	if (ect->filter) {
-		edit->old_text = (*ect->filter)(e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-	} else {
-		edit->old_text = g_strdup (e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-	}
+	temp = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	edit->old_text = g_strdup (temp);
+	ect_free_text(ect, temp);
 	edit->cell.text = g_strdup (edit->old_text);
 
 #if 0
@@ -1230,11 +1039,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	GnomeFont *font = gnome_font_new ("Helvetica", 12);
 	char *string;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
-	if (ect->filter) {
-		string = (*ect->filter)(e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-	} else {
-		string = e_table_model_value_at (ecell_view->e_table_model, model_col, row);
-	}
+	string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
 	gnome_print_gsave(context);
 	if (gnome_print_moveto(context, 2, 2) == -1)
 				/* FIXME */;
@@ -1252,9 +1057,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	gnome_print_setfont(context, font);
 	gnome_print_show(context, string);
 	gnome_print_grestore(context);
-	if (ect->filter) {
-		g_free(string);
-	}
+	ect_free_text(ect, string);
 }
 
 static gdouble
@@ -1391,6 +1194,8 @@ ect_show_tooltip (ECellView *ecell_view,
 
 	if (!cut_off) {
 		tooltip->timer = 0;
+		unref_lines (&cell);
+		unbuild_current_cell (&cell);
 		return;
 	}
 
@@ -1442,6 +1247,7 @@ ect_show_tooltip (ECellView *ecell_view,
 					      "clip", TRUE,
 					      "line_wrap", FALSE,
   					      "justification", E_CELL_TEXT (text_view->cell_view.ecell)->justify,
+					      "draw_background", FALSE,
 					      NULL);
 
 	tooltip_width = max_width;
@@ -1518,9 +1324,6 @@ ect_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		text->color_column = GTK_VALUE_INT (*arg);
 		break;
 
-	case ARG_TEXT_FILTER:
-		text->filter = GTK_VALUE_POINTER (*arg);
-		break;
 	default:
 		return;
 	}
@@ -1547,10 +1350,6 @@ ect_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_INT (*arg) = text->color_column;
 		break;
 
-	case ARG_TEXT_FILTER:
-		GTK_VALUE_POINTER (*arg) = text->filter;
-		break;
-
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -1561,6 +1360,7 @@ static void
 e_cell_text_class_init (GtkObjectClass *object_class)
 {
 	ECellClass *ecc = (ECellClass *) object_class;
+	ECellTextClass *ectc = (ECellTextClass *) object_class;
 
 	object_class->destroy = ect_destroy;
 
@@ -1578,6 +1378,9 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 	ecc->max_width = ect_max_width;
 	ecc->show_tooltip = ect_show_tooltip;
 
+	ectc->get_text = ect_real_get_text;
+	ectc->free_text = ect_real_free_text;
+
 	object_class->get_arg = ect_get_arg;
 	object_class->set_arg = ect_set_arg;
 
@@ -1589,8 +1392,6 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_BOLD_COLUMN);
 	gtk_object_add_arg_type ("ECellText::color_column",
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_COLOR_COLUMN);
-	gtk_object_add_arg_type ("ECellText::text_filter",
-				 GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_TEXT_FILTER);
 
 	if (!clipboard_atom)
 		clipboard_atom = gdk_atom_intern ("CLIPBOARD", FALSE);
@@ -1599,6 +1400,8 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 static void
 e_cell_text_init (ECellText *ect)
 {
+	ect->ellipsis = NULL;
+	ect->use_ellipsis = TRUE;
 	ect->strikeout_column = -1;
 	ect->bold_column = -1;
 	ect->color_column = -1;
@@ -1606,6 +1409,25 @@ e_cell_text_init (ECellText *ect)
 
 E_MAKE_TYPE(e_cell_text, "ECellText", ECellText, e_cell_text_class_init, e_cell_text_init, PARENT_TYPE);
 
+/**
+ * e_cell_text_construct:
+ * @cell: The cell to construct
+ * @fontname: font to be used to render on the screen
+ * @justify: Justification of the string in the cell
+ *
+ * constructs the ECellText.  To be used by subclasses and language
+ * bindings.
+ *
+ * Returns: The ECellText.
+ */
+ECell *
+e_cell_text_construct (ECellText *cell, const char *fontname, GtkJustification justify)
+{
+	cell->font_name = g_strdup (fontname);
+	cell->justify = justify;
+	return E_CELL(cell);
+}
+
 /**
  * e_cell_text_new:
  * @fontname: font to be used to render on the screen
@@ -1618,7 +1440,7 @@ E_MAKE_TYPE(e_cell_text, "ECellText", ECellText, e_cell_text_class_init, e_cell_
  * The ECellText object support a large set of properties that can be
  * configured through the Gtk argument system and allows the user to have
  * a finer control of the way the string is displayed.  The arguments supported
- * allow the control of strikeout, bold, color and a text filter.
+ * allow the control of strikeout, bold, and color.
  *
  * The arguments "strikeout_column", "bold_column" and "color_column" set
  * and return an integer that points to a column in the model that controls
@@ -1635,12 +1457,8 @@ e_cell_text_new (const char *fontname, GtkJustification justify)
 {
 	ECellText *ect = gtk_type_new (e_cell_text_get_type ());
 
-	ect->ellipsis = NULL;
-	ect->use_ellipsis = TRUE;
+	e_cell_text_construct(ect, fontname, justify);
 
-	ect->font_name = g_strdup (fontname);
-	ect->justify = justify;
-      
 	return (ECell *) ect;
 }
 
@@ -2399,7 +2217,6 @@ split_into_lines (CurrentCell *cell)
 	linebreaks = cell->breaks;
 
 	/* Check if already split. */
-
 	linebreaks->lines = NULL;
 	linebreaks->num_lines = 0;
 	
@@ -2534,6 +2351,7 @@ build_current_cell (CurrentCell *cell, ECellTextView *text_view, int model_col,
 {
 	ECellView *ecell_view = (ECellView *) text_view;
 	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
+	char *temp;
 
 	cell->text_view = text_view;
 	cell->model_col = model_col;
@@ -2541,11 +2359,10 @@ build_current_cell (CurrentCell *cell, ECellTextView *text_view, int model_col,
 	cell->row = row;
 	cell->breaks = NULL;
 
-	if (ect->filter) {
-		cell->text = (*ect->filter)(e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-	} else {
-		cell->text = g_strdup (e_table_model_value_at (ecell_view->e_table_model, model_col, row));
-	}
+	temp = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	cell->text = g_strdup(temp);
+	ect_free_text(ect, temp);
+
 	cell->width = e_table_header_get_column (
 		((ETableItem *)ecell_view->e_table_item_view)->header,
 		view_col)->width - 8;
