@@ -1,14 +1,14 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* ECellText - Text item for e-table.
- * Copyright (C) 2000 Helix Code, Inc.
- * Author: Chris Lahey <clahey@helixcode.com>
+ * Copyright (C) 2000 Ximian, Inc.
+ * Author: Chris Lahey <clahey@ximian.com>
  *
  * Drawing and event handling from:
  * 
  * EText - Text item for evolution.
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Author: Chris Lahey <clahey@helixcode.com>
+ * Author: Chris Lahey <clahey@ximian.com>
  *
  * A majority of code taken from:
  *
@@ -27,9 +27,6 @@
 #include <libgnomeui/gnome-canvas.h>
 #include <gal/e-table/e-cell.h>
 
-/* Should return a malloced object. */
-typedef char *(*ECellTextFilter) (const void *);
-
 #define E_CELL_TEXT_TYPE        (e_cell_text_get_type ())
 #define E_CELL_TEXT(o)          (GTK_CHECK_CAST ((o), E_CELL_TEXT_TYPE, ECellText))
 #define E_CELL_TEXT_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_CELL_TEXT_TYPE, ECellTextClass))
@@ -59,18 +56,20 @@ typedef struct {
 	   See the XParseColor man page for the formats available. */
 	int color_column;
 
-	ECellTextFilter filter;
-
 	/* This stores the colors we have allocated. */
 	GHashTable *colors;
 } ECellText;
 
 typedef struct {
 	ECellClass parent_class;
+
+	char *(*get_text)  (ECellText *cell, ETableModel *model, int col, int row);
+	void  (*free_text) (ECellText *cell, char *text);
 } ECellTextClass;
 
 GtkType    e_cell_text_get_type (void);
 ECell     *e_cell_text_new      (const char *fontname, GtkJustification justify);
+ECell     *e_cell_text_construct(ECellText *cell, const char *fontname, GtkJustification justify);
 
 #endif /* _E_CELL_TEXT_H_ */
 
