@@ -3,10 +3,10 @@
  * E-Table-Group.c: Implements the grouping objects for elements on a table
  *
  * Author:
- *   Chris Lahey <clahey@helixcode.com>
+ *   Chris Lahey <clahey@ximian.com>
  *   Miguel de Icaza (miguel@gnu.org)
  *
- * Copyright 1999, 2000 Helix Code, Inc.
+ * Copyright 1999, 2000 Ximian, Inc.
  */
 
 #include <config.h>
@@ -109,6 +109,18 @@ etgc_destroy (GtkObject *object)
 	GTK_OBJECT_CLASS (etgc_parent_class)->destroy (object);
 }
 
+/** 
+ * e_table_group_container_construct
+ * @parent: The %GnomeCanvasGroup to create a child of.
+ * @etgc: The %ETableGroupContainer.
+ * @full_header: The full header of the %ETable.
+ * @header: The current header of the %ETable.
+ * @model: The %ETableModel of the %ETable.
+ * @sort_info: The %ETableSortInfo of the %ETable.
+ * @n: Which grouping level this is (Starts at 0 and sends n + 1 to any child %ETableGroups.
+ *
+ * This routine constructs the new %ETableGroupContainer.
+ */
 void
 e_table_group_container_construct (GnomeCanvasGroup *parent, ETableGroupContainer *etgc,
 				   ETableHeader *full_header,
@@ -118,10 +130,9 @@ e_table_group_container_construct (GnomeCanvasGroup *parent, ETableGroupContaine
 	ETableCol *col;
 	ETableSortColumn column = e_table_sort_info_grouping_get_nth(sort_info, n);
 
-	if (column.column > e_table_header_count (full_header))
+	col = e_table_header_get_column_by_col_idx(full_header, column.column);
+	if (col == NULL)
 		col = e_table_header_get_column (full_header, e_table_header_count (full_header) - 1);
-	else
-		col = e_table_header_get_column (full_header, column.column);
 
 	e_table_group_construct (parent, E_TABLE_GROUP (etgc), full_header, header, model);
 	etgc->ecol = col;
@@ -141,6 +152,21 @@ e_table_group_container_construct (GnomeCanvasGroup *parent, ETableGroupContaine
 	etgc->open = TRUE;
 }
 
+/** 
+ * e_table_group_container_new
+ * @parent: The %GnomeCanvasGroup to create a child of.
+ * @full_header: The full header of the %ETable.
+ * @header: The current header of the %ETable.
+ * @model: The %ETableModel of the %ETable.
+ * @sort_info: The %ETableSortInfo of the %ETable.
+ * @n: Which grouping level this is (Starts at 0 and sends n + 1 to any child %ETableGroups.
+ *
+ * %ETableGroupContainer is an %ETableGroup which groups by the nth
+ * grouping of the %ETableSortInfo.  It creates %ETableGroups as
+ * children.
+ *
+ * Returns: The new %ETableGroupContainer.
+ */
 ETableGroup *
 e_table_group_container_new (GnomeCanvasGroup *parent, ETableHeader *full_header,
 			     ETableHeader     *header,
@@ -326,6 +352,13 @@ child_cursor_change (ETableGroup *etg, int row,
 	e_table_group_cursor_change (E_TABLE_GROUP (etgc), row);
 }
 
+static void
+child_cursor_activated (ETableGroup *etg, int row,
+		    ETableGroupContainer *etgc)
+{
+	e_table_group_cursor_activated (E_TABLE_GROUP (etgc), row);
+}
+
 static void
 child_double_click (ETableGroup *etg, int row, int col, GdkEvent *event,
 		    ETableGroupContainer *etgc)
@@ -392,6 +425,7 @@ etgc_add (ETableGroup *etg, gint row)
 						  "font_gdk", etgc->font,
 						  "anchor", GTK_ANCHOR_SW,
 						  "fill_color", "black",
+						  "draw_background", FALSE,
 						  NULL);
 	child = e_table_group_new (GNOME_CANVAS_GROUP (etgc), etg->full_header,
 				   etg->header, etg->model, etgc->sort_info, etgc->n + 1);
@@ -401,9 +435,13 @@ etgc_add (ETableGroup *etg, gint row)
 			      "cursor_mode", etgc->cursor_mode,
 			      "table_selection_model", etgc->table_selection_model,
 			      "length_threshold", etgc->length_threshold,
+			      "minimum_width", etgc->minimum_width - GROUP_INDENT,
 			      NULL);
+
 	gtk_signal_connect (GTK_OBJECT (child), "cursor_change",
 			    GTK_SIGNAL_FUNC (child_cursor_change), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "cursor_activated",
+			    GTK_SIGNAL_FUNC (child_cursor_activated), etgc);
 	gtk_signal_connect (GTK_OBJECT (child), "double_click",
 			    GTK_SIGNAL_FUNC (child_double_click), etgc);
 	gtk_signal_connect (GTK_OBJECT (child), "right_click",
@@ -824,11 +862,16 @@ e_table_group_apply_to_leafs (ETableGroup *etg, ETableGroupLeafFn fn, void *clos
 		ETableGroupContainer *etgc = E_TABLE_GROUP_CONTAINER (etg);
 		GList *list = etgc->children;
 
+		/* Protect from unrefs in the callback functions */
+		gtk_object_ref (GTK_OBJECT (etg));
+
 		for (list = etgc->children; list; list = list->next){
 			ETableGroupContainerChildNode *child_node = list->data;
 
 			e_table_group_apply_to_leafs (child_node->child, fn, closure);
 		}
+
+		gtk_object_unref (GTK_OBJECT (etg));
 	} else if (E_IS_TABLE_GROUP_LEAF (etg)){
 		(*fn) (E_TABLE_GROUP_LEAF (etg)->item, closure);
 	} else {
