@@ -5,11 +5,12 @@
  * Author:
  *   Miguel de Icaza (miguel@gnu.org)
  *
- * Copyright 1999, 2000 Helix Code, Inc.
+ * Copyright 1999, 2000, 2001 Ximian, Inc.
  */
 #include <config.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkdnd.h>
+#include "gal/util/e-i18n.h"
 #include <libgnomeui/gnome-canvas.h>
 #include <libgnomeui/gnome-canvas-util.h>
 #include <libgnomeui/gnome-canvas-polygon.h>
@@ -102,6 +103,9 @@ ethi_destroy (GtkObject *object){
 	if (ethi->full_header)
 		gtk_object_unref (GTK_OBJECT(ethi->full_header));
 
+	if (ethi->config)
+		gtk_object_destroy (GTK_OBJECT (ethi->config));
+	
 	if (GTK_OBJECT_CLASS (ethi_parent_class)->destroy)
 		(*GTK_OBJECT_CLASS (ethi_parent_class)->destroy) (object);
 }
@@ -784,8 +788,8 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			g_hash_table_insert (arrows, 
 					     (gpointer) column.column,
 					     (gpointer) (column.ascending ?
-							 E_TABLE_COL_ARROW_DOWN : 
-							 E_TABLE_COL_ARROW_UP));
+							 E_TABLE_COL_ARROW_UP : 
+							 E_TABLE_COL_ARROW_DOWN));
 		}
 		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
 		for (i = 0; i < length; i++) {
@@ -793,8 +797,8 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			g_hash_table_insert (arrows, 
 					     (gpointer) column.column,
 					     (gpointer) (column.ascending ?
-							 E_TABLE_COL_ARROW_DOWN : 
-							 E_TABLE_COL_ARROW_UP));
+							 E_TABLE_COL_ARROW_UP : 
+							 E_TABLE_COL_ARROW_DOWN));
 		}
 	}
 
@@ -969,27 +973,36 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 		int length = e_table_sort_info_grouping_get_count(ethi->sort_info);
 		int i;
 		for (i = 0; i < length; i++) {
-			ETableSortColumn column = e_table_sort_info_grouping_get_nth(ethi->sort_info, i);
+			ETableSortColumn column =
+				e_table_sort_info_grouping_get_nth(
+					ethi->sort_info, i);
 			group_indent ++;
-			g_hash_table_insert (arrows, 
-					     (gpointer) column.column,
-					     (gpointer) (column.ascending ?
-							 E_TABLE_COL_ARROW_DOWN : 
-							 E_TABLE_COL_ARROW_UP));
+			g_hash_table_insert (
+				arrows, 
+				(gpointer) column.column,
+				(gpointer) (column.ascending ?
+					    E_TABLE_COL_ARROW_UP : 
+					    E_TABLE_COL_ARROW_DOWN));
 		}
 		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
 		for (i = 0; i < length; i++) {
-			ETableSortColumn column = e_table_sort_info_sorting_get_nth(ethi->sort_info, i);
-			g_hash_table_insert (arrows, 
-					     (gpointer) column.column,
-					     (gpointer) (column.ascending ?
-					     E_TABLE_COL_ARROW_DOWN : 
-							 E_TABLE_COL_ARROW_UP));
+			ETableSortColumn column =
+				e_table_sort_info_sorting_get_nth (
+					ethi->sort_info, i);
+
+			g_hash_table_insert (
+				arrows, 
+				(gpointer) column.column,
+				(gpointer) (column.ascending ?
+					    E_TABLE_COL_ARROW_UP : 
+					    E_TABLE_COL_ARROW_DOWN));
 		}
 	}
 
-	ethi_drag_types[0].target = g_strdup_printf("%s-%s", ethi_drag_types[0].target, ethi->dnd_code);
-	list = gtk_target_list_new (ethi_drag_types, ELEMENTS (ethi_drag_types));
+	ethi_drag_types[0].target = g_strdup_printf(
+		"%s-%s", ethi_drag_types[0].target, ethi->dnd_code);
+	list = gtk_target_list_new (
+		ethi_drag_types, ELEMENTS (ethi_drag_types));
 	context = gtk_drag_begin (widget, list, GDK_ACTION_MOVE, 1, event);
 	g_free(ethi_drag_types[0].target);
 
@@ -997,21 +1010,23 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 	col_width = ecol->width;
 	pixmap = gdk_pixmap_new (widget->window, col_width, ethi->height, -1);
 
-	e_table_header_draw_button (pixmap, ecol,
-				    widget->style, ethi->font,
-				    GTK_WIDGET_STATE (widget),
-				    widget, ethi->gc,
-				    0, 0,
-				    col_width, ethi->height,
-				    col_width, ethi->height,
-				    (ETableColArrow) g_hash_table_lookup (
-					    arrows, (gpointer) ecol->col_idx));
-	gtk_drag_set_icon_pixmap        (context,
-					 gdk_window_get_colormap (widget->window),
-					 pixmap,
-					 NULL,
-					 col_width / 2,
-					 ethi->height / 2);
+	e_table_header_draw_button (
+		pixmap, ecol,
+		widget->style, ethi->font,
+		GTK_WIDGET_STATE (widget),
+		widget, ethi->gc,
+		0, 0,
+		col_width, ethi->height,
+		col_width, ethi->height,
+		(ETableColArrow) g_hash_table_lookup (
+			arrows, (gpointer) ecol->col_idx));
+	gtk_drag_set_icon_pixmap (
+		context,
+		gdk_window_get_colormap (widget->window),
+		pixmap,
+		NULL,
+		col_width / 2,
+		ethi->height / 2);
 	gdk_pixmap_unref (pixmap);
 
 	ethi->maybe_drag = FALSE;
@@ -1038,21 +1053,28 @@ ethi_popup_sort_ascending(GtkWidget *widget, EthiHeaderInfo *info)
 
 	length = e_table_sort_info_grouping_get_count(ethi->sort_info);
 	for (i = 0; i < length; i++) {
-		ETableSortColumn column = e_table_sort_info_grouping_get_nth(ethi->sort_info, i);
+		ETableSortColumn column = e_table_sort_info_grouping_get_nth (
+			ethi->sort_info, i);
+
 		if (model_col == column.column){
 			column.ascending = 1;
-			e_table_sort_info_grouping_set_nth(ethi->sort_info, i, column);
+			e_table_sort_info_grouping_set_nth (
+				ethi->sort_info, i, column);
 			found = 1;
 			break;
 		}
 	}
 	if (!found) {
-		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
+		length = e_table_sort_info_sorting_get_count (
+			ethi->sort_info);
 		for (i = 0; i < length; i++) {
-			ETableSortColumn column = e_table_sort_info_sorting_get_nth(ethi->sort_info, i);
+			ETableSortColumn column =
+				e_table_sort_info_sorting_get_nth(
+					ethi->sort_info, i);
 			if (model_col == column.column){
 				column.ascending = 1;
-				e_table_sort_info_sorting_set_nth(ethi->sort_info, i, column);
+				e_table_sort_info_sorting_set_nth (
+					ethi->sort_info, i, column);
 				found = 1;
 				break;
 			}
@@ -1082,21 +1104,27 @@ ethi_popup_sort_descending(GtkWidget *widget, EthiHeaderInfo *info)
 
 	length = e_table_sort_info_grouping_get_count(ethi->sort_info);
 	for (i = 0; i < length; i++) {
-		ETableSortColumn column = e_table_sort_info_grouping_get_nth(ethi->sort_info, i);
+		ETableSortColumn column = e_table_sort_info_grouping_get_nth(
+			ethi->sort_info, i);
 		if (model_col == column.column){
 			column.ascending = 0;
-			e_table_sort_info_grouping_set_nth(ethi->sort_info, i, column);
+			e_table_sort_info_grouping_set_nth(
+				ethi->sort_info, i, column);
 			found = 1;
 			break;
 		}
 	}
 	if (!found) {
-		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
+		length = e_table_sort_info_sorting_get_count (ethi->sort_info);
 		for (i = 0; i < length; i++) {
-			ETableSortColumn column = e_table_sort_info_sorting_get_nth(ethi->sort_info, i);
+			ETableSortColumn column =
+				e_table_sort_info_sorting_get_nth(
+					ethi->sort_info, i);
+
 			if (model_col == column.column){
 				column.ascending = 0;
-				e_table_sort_info_sorting_set_nth(ethi->sort_info, i, column);
+				e_table_sort_info_sorting_set_nth (
+					ethi->sort_info, i, column);
 				found = 1;
 				break;
 			}
@@ -1104,10 +1132,11 @@ ethi_popup_sort_descending(GtkWidget *widget, EthiHeaderInfo *info)
 	}
 	if (!found) {
 		ETableSortColumn column = { model_col, 0 };
-		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
+		length = e_table_sort_info_sorting_get_count (ethi->sort_info);
 		if (length == 0)
 			length++;
-		e_table_sort_info_sorting_set_nth(ethi->sort_info, length - 1, column);
+		e_table_sort_info_sorting_set_nth (
+			ethi->sort_info, length - 1, column);
 	}
 }
 
@@ -1185,6 +1214,25 @@ ethi_popup_format_columns(GtkWidget *widget, EthiHeaderInfo *info)
 {
 }
 
+static void
+config_destroyed (GtkObject *object, ETableHeaderItem *ethi)
+{
+	ethi->config = NULL;
+}
+
+static void
+apply_changes (GnomePropertyBox *pbox, gint page_num, ETableHeaderItem *ethi)
+{
+	ETableConfig *config = E_TABLE_CONFIG (ethi->config);
+	char *state = e_table_state_save_to_string (config->state);
+
+	if (page_num != -1)
+		return;
+
+	e_table_set_state (ethi->table, state);
+	g_free (state);
+}
+
 static void
 ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 {
@@ -1192,15 +1240,21 @@ ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 	ETableState *state;
 
 	if (ethi->config)
-		gdk_window_raise(GTK_WIDGET(ethi->config)->window);
+		e_table_config_raise (E_TABLE_CONFIG (ethi->config));
 	else {
 		state = e_table_get_state_object(ethi->table);
 
-		ethi->config = e_table_config_new(ethi->table->spec,
-						  state);
-		gtk_signal_connect(GTK_OBJECT(ethi->config), "clicked",
-				   GTK_SIGNAL_FUNC(gnome_dialog_close), ethi);
-		gtk_widget_show(ethi->config);
+		ethi->config = e_table_config_new (
+			_("Configuring view: FIXME"),
+			ethi->table->spec, state);
+		gtk_signal_connect (
+			GTK_OBJECT (ethi->config), "destroy",
+			GTK_SIGNAL_FUNC (config_destroyed), ethi);
+		gtk_signal_connect (
+			GTK_OBJECT (
+				E_TABLE_CONFIG (ethi->config)->dialog_toplevel),
+			"apply",
+			GTK_SIGNAL_FUNC (apply_changes), ethi);
 	}
 }
 
@@ -1216,7 +1270,7 @@ static EPopupMenu ethi_context_menu [] = {
 	{ N_("Group By Box"),              NULL, GTK_SIGNAL_FUNC(ethi_popup_group_box),       NULL, 1},
 	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 1},
 	{ N_("Remove This Column"),        NULL, GTK_SIGNAL_FUNC(ethi_popup_remove_column),   NULL, 0},
-	{ N_("Field Chooser"),             NULL, GTK_SIGNAL_FUNC(ethi_popup_field_chooser),   NULL, 0},
+	{ N_("Add a Column..."),           NULL, GTK_SIGNAL_FUNC(ethi_popup_field_chooser),   NULL, 0},
 	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 1},
 	{ N_("Alignment"),                 NULL, GTK_SIGNAL_FUNC(ethi_popup_alignment),       NULL, 1},
 	{ N_("Best Fit"),                  NULL, GTK_SIGNAL_FUNC(ethi_popup_best_fit),        NULL, 2},
@@ -1234,7 +1288,7 @@ ethi_header_context_menu (ETableHeaderItem *ethi, GdkEventButton *event)
 	info->ethi = ethi;
 	info->col = ethi_find_col_by_x (ethi, event->x);
 	col = e_table_header_get_column (ethi->eth, info->col);
-	e_popup_menu_run (ethi_context_menu, event,
+	e_popup_menu_run (ethi_context_menu, (GdkEvent *) event,
 			  1 +
 			  (col->sortable ? 0 : 2) +
 			  (ethi->table ? 0 : 4), 0, info);
@@ -1247,6 +1301,68 @@ ethi_button_pressed (ETableHeaderItem *ethi, GdkEventButton *event)
 			 ethi_signals [BUTTON_PRESSED], event);
 }
 
+static void
+ethi_change_sort_state (ETableHeaderItem *ethi, gdouble x)
+{
+	ETableCol *col;
+	int model_col;
+	int length;
+	int i;
+	int found = FALSE;
+	
+	col = e_table_header_get_column (ethi->eth, ethi_find_col_by_x (ethi, x));
+	model_col = col->col_idx;
+	
+	length = e_table_sort_info_grouping_get_count(ethi->sort_info);
+	for (i = 0; i < length; i++) {
+		ETableSortColumn column = e_table_sort_info_grouping_get_nth(ethi->sort_info, i);
+		if (model_col == column.column){
+			int ascending = column.ascending;
+			ascending = ! ascending;
+			column.ascending = ascending;
+			e_table_sort_info_grouping_set_nth(ethi->sort_info, i, column);
+			found = 1;
+			break;
+		}
+	}
+	
+	if (!col->sortable)
+		return;
+	
+	if (!found) {
+		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
+		for (i = 0; i < length; i++) {
+			ETableSortColumn column = e_table_sort_info_sorting_get_nth(ethi->sort_info, i);
+
+			if (model_col == column.column){
+				int ascending = column.ascending;
+				
+				if (ascending == 0){
+					/*
+					 * This means the user has clicked twice
+					 * already, lets kill sorting now.
+					 */
+					e_table_sort_info_sorting_truncate (ethi->sort_info, i);
+				} else {
+					ascending = !ascending;
+					column.ascending = ascending;
+					e_table_sort_info_sorting_set_nth(ethi->sort_info, i, column);
+				}
+				found = 1;
+				break;
+			}
+		}
+	}
+
+	if (!found) {
+		ETableSortColumn column = { model_col, 1 };
+		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
+		if (length == 0)
+			length++;
+		e_table_sort_info_sorting_set_nth(ethi->sort_info, length - 1, column);
+	}
+}
+
 /*
  * Handles the events on the ETableHeaderItem, particularly it handles resizing
  */
@@ -1362,52 +1478,8 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 		if (ethi->resize_col != -1){
 			needs_ungrab = (ethi->resize_guide != NULL);
 			ethi_end_resize (ethi);
-		} else if (was_maybe_drag && ethi->sort_info) {
-			ETableCol *col;
-			int model_col;
-			int length;
-			int i;
-			int found = FALSE;
-
-			col = e_table_header_get_column (ethi->eth, ethi_find_col_by_x (ethi, e->button.x));
-			model_col = col->col_idx;
-
-			length = e_table_sort_info_grouping_get_count(ethi->sort_info);
-			for (i = 0; i < length; i++) {
-				ETableSortColumn column = e_table_sort_info_grouping_get_nth(ethi->sort_info, i);
-				if (model_col == column.column){
-					int ascending = column.ascending;
-					ascending = ! ascending;
-					column.ascending = ascending;
-					e_table_sort_info_grouping_set_nth(ethi->sort_info, i, column);
-					found = 1;
-					break;
-				}
-			}
-			if(col->sortable) {
-				if (!found) {
-					length = e_table_sort_info_sorting_get_count(ethi->sort_info);
-					for (i = 0; i < length; i++) {
-						ETableSortColumn column = e_table_sort_info_sorting_get_nth(ethi->sort_info, i);
-						if (model_col == column.column){
-							int ascending = column.ascending;
-							ascending = ! ascending;
-							column.ascending = ascending;
-							e_table_sort_info_sorting_set_nth(ethi->sort_info, i, column);
-							found = 1;
-							break;
-						}
-					}
-				}
-				if (!found) {
-					ETableSortColumn column = { model_col, 1 };
-					length = e_table_sort_info_sorting_get_count(ethi->sort_info);
-					if (length == 0)
-						length++;
-					e_table_sort_info_sorting_set_nth(ethi->sort_info, length - 1, column);
-				}
-			}
-		}
+		} else if (was_maybe_drag && ethi->sort_info) 
+			ethi_change_sort_state (ethi, e->button.x);
 		
 		if (needs_ungrab)
 			gnome_canvas_item_ungrab (item, e->button.time);
