@@ -3,9 +3,10 @@
  * E-table-item.c: A GnomeCanvasItem that is a view of an ETableModel.
  *
  * Author:
+ *   Christopher James Lahey <clahey@ximian.com>
  *   Miguel de Icaza (miguel@gnu.org)
  *
- * Copyright 1999, Helix Code, Inc.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
  * TODO:
  *   Add a border to the thing, so that focusing works properly.
@@ -31,6 +32,7 @@ static GnomeCanvasItemClass *eti_parent_class;
 
 enum {
 	CURSOR_CHANGE,
+	CURSOR_ACTIVATED,
 	DOUBLE_CLICK,
 	RIGHT_CLICK,
 	CLICK,
@@ -56,11 +58,16 @@ enum {
 	ARG_HEIGHT,
 };
 
+#define DOUBLE_CLICK_TIME      250
+#define TRIPLE_CLICK_TIME      500
+
+
 static int eti_get_height (ETableItem *eti);
 static int eti_get_minimum_width (ETableItem *eti);
 static int eti_row_height (ETableItem *eti, int row);
 static void e_table_item_focus (ETableItem *eti, int col, int row, GdkModifierType state);
 static void eti_cursor_change (ETableSelectionModel *selection, int row, int col, ETableItem *eti);
+static void eti_cursor_activated (ETableSelectionModel *selection, int row, int col, ETableItem *eti);
 static void eti_selection_change (ETableSelectionModel *selection, ETableItem *eti);
 #if 0
 static void eti_request_region_show (ETableItem *eti,
@@ -118,7 +125,7 @@ inline static gint
 view_to_model_col(ETableItem *eti, int col)
 {
 	ETableCol *ecol = e_table_header_get_column (eti->header, col);
-	return ecol->col_idx;
+	return ecol ? ecol->col_idx : -1;
 }
 
 static gboolean
@@ -345,10 +352,12 @@ eti_remove_table_selection_model (ETableItem *eti)
 			       eti->selection_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
 			       eti->cursor_change_id);
+	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
+			       eti->cursor_activated_id);
 	gtk_object_unref (GTK_OBJECT (eti->selection));
 
 	eti->selection_change_id = 0;
-	eti->cursor_change_id = 0;
+	eti->cursor_activated_id = 0;
 	eti->selection = NULL;
 }
 
@@ -726,6 +735,17 @@ eti_table_model_row_deleted (ETableModel *table_model, int row, ETableItem *eti)
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (eti));
 }
 
+/** 
+ * e_table_item_redraw_range
+ * @eti: %ETableItem which will be redrawn
+ * @start_col: The first col to redraw.
+ * @start_row: The first row to redraw.
+ * @end_col: The last col to redraw.
+ * @end_row: The last row to redraw.
+ *
+ * This routine redraws the given %ETableItem in the range given.  The
+ * range is inclusive at both ends.
+ */
 void
 e_table_item_redraw_range (ETableItem *eti,
 			   int start_col, int start_row,
@@ -816,6 +836,10 @@ eti_add_table_selection_model (ETableItem *eti, ETableSelectionModel *selection)
 		GTK_OBJECT (selection), "cursor_changed",
 		GTK_SIGNAL_FUNC (eti_cursor_change), eti);
 
+	eti->cursor_activated_id = gtk_signal_connect (
+		GTK_OBJECT (selection), "cursor_activated",
+		GTK_SIGNAL_FUNC (eti_cursor_activated), eti);
+
 	eti_selection_change(selection, eti);
 }
 
@@ -1037,11 +1061,14 @@ eti_init (GnomeCanvasItem *item)
 
 	eti->selection_change_id       = 0;
 	eti->cursor_change_id          = 0;
+	eti->cursor_activated_id          = 0;
 	eti->selection                 = NULL;
 
 	eti->needs_redraw              = 0;
 	eti->needs_compute_height      = 0;
 
+	eti->in_key_press              = 0;
+
 	eti->tooltip                   = g_new0 (ETableTooltip, 1);
 	eti->tooltip->timer            = 0;
 	eti->tooltip->window           = NULL;
@@ -1166,7 +1193,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	int f_x1, f_x2, f_y1, f_y2;
 	gboolean f_found;
 	double i2c [6];
-	ArtPoint eti_base, eti_base_item;
+	ArtPoint eti_base, eti_base_item, lower_right;
 	GtkWidget *canvas = GTK_WIDGET(item->canvas);
 	GdkColor *background;
 	
@@ -1187,6 +1214,10 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	eti_base_item.y = eti->y1;
 	art_affine_point (&eti_base, &eti_base_item, i2c);
 
+	eti_base_item.x = eti->x1 + eti->width;
+	eti_base_item.y = eti->y1 + eti->height;
+	art_affine_point (&lower_right, &eti_base_item, i2c);
+
 	/*
 	 * First column to draw, last column to draw
 	 */
@@ -1279,6 +1310,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 			ECellFlags flags;
 			switch (eti->cursor_mode) {
 			case E_TABLE_CURSOR_SIMPLE:
+			case E_TABLE_CURSOR_SPREADSHEET:
 				if (cursor_col == ecol->col_idx && cursor_row == view_to_model_row(eti, row))
 					col_selected = !col_selected;
 				break;
@@ -1327,12 +1359,28 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 			e_cell_draw (ecell_view, drawable, ecol->col_idx, col, row, flags,
 				     xd, yd, xd + ecol->width, yd + height);
 			
-			if (view_to_model_col(eti, col) == cursor_col && view_to_model_row(eti, row) == cursor_row){
-				f_x1 = xd;
-				f_x2 = xd + ecol->width;
-				f_y1 = yd;
-				f_y2 = yd + height;
-				f_found = TRUE;
+			if (!f_found) {
+				switch (eti->cursor_mode) {
+				case E_TABLE_CURSOR_LINE:
+					if (view_to_model_row(eti, row) == cursor_row) {
+						f_x1 = floor (eti_base.x) - x;
+						f_x2 = floor (lower_right.x) - x;
+						f_y1 = yd;
+						f_y2 = yd + height;
+						f_found = TRUE;
+					}
+					break;
+				case E_TABLE_CURSOR_SIMPLE:
+				case E_TABLE_CURSOR_SPREADSHEET:
+					if (view_to_model_col(eti, col) == cursor_col && view_to_model_row(eti, row) == cursor_row) {
+						f_x1 = xd;
+						f_x2 = xd + ecol->width;
+						f_y1 = yd;
+						f_y2 = yd + height;
+						f_found = TRUE;
+					}
+					break;
+				}
 			}
 
 			xd += ecol->width;
@@ -1367,7 +1415,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	/*
 	 * Draw focus
 	 */
-	if (f_found && eti->draw_focus){
+	if (eti->draw_focus && f_found) {
 		gdk_gc_set_ts_origin (eti->focus_gc, f_x1, f_y1);
 		gdk_draw_rectangle (drawable, eti->focus_gc, FALSE,
 				    f_x1, f_y1, f_x2 - f_x1 - 1, f_y2 - f_y1 - 1);
@@ -1470,6 +1518,7 @@ eti_cursor_move_right (ETableItem *eti)
 	eti_cursor_move (eti, model_to_view_row(eti, cursor_row), model_to_view_col(eti, cursor_col) + 1);
 }
 
+#if 0
 static void
 eti_cursor_move_up (ETableItem *eti)
 {
@@ -1493,7 +1542,9 @@ eti_cursor_move_down (ETableItem *eti)
 
 	eti_cursor_move (eti, model_to_view_row(eti, cursor_row) + 1, model_to_view_col(eti, cursor_col));
 }
+#endif
 
+#if 0
 static int
 _do_tooltip (ETableItem *eti)
 {
@@ -1528,6 +1579,7 @@ _do_tooltip (ETableItem *eti)
 			     eti->tooltip);
 	return FALSE;
 }
+#endif
 
 static gint
 eti_e_cell_event     (ETableItem *item, ECellView *ecell_view, GdkEvent *event, int time, int model_col, int view_col, int row, ECellFlags flags)
@@ -1571,15 +1623,18 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			gtk_timeout_remove (eti->tooltip->timer);
 			eti->tooltip->timer = 0;
 		}
-		e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti));
 
 		switch (e->button.button) {
 		case 1: /* Fall through. */
 		case 2:
+			e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti));
 			gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
 
-			if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
+			if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1)) {
+				if (eti_editing (eti))
+					e_table_item_leave_edit (eti);
 				return TRUE;
+			}
 
 			ecell_view = eti->cell_views [col];
 			button = *(GdkEventButton *)e;
@@ -1616,9 +1671,11 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, 
 							       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
 			}
+			g_print("Single click\n");
 
 			break;
 		case 3:
+			e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti));
 			gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
 			if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
 				return TRUE;
@@ -1676,6 +1733,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				return_val = eti_e_cell_event (eti, ecell_view, e, e->button.time,
 							       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
 			}
+			g_print ("Release\n");
 			break;
 		case 3:
 		case 4:
@@ -1688,7 +1746,6 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	}
 
 	case GDK_2BUTTON_PRESS: {
-		double x1, y1;
 		int col, row;
 		GdkEventButton button;
 
@@ -1698,15 +1755,24 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 		gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
 
+#if 0
 		if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
 			return TRUE;
+#endif
+		gtk_object_get(GTK_OBJECT(eti->selection),
+			       "cursor_row", &row,
+			       "cursor_col", &col,
+			       NULL);
 
+#if 0
 		button = *(GdkEventButton *)e;
 		button.x = x1;
 		button.y = y1;
+#endif
 
 		gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
-				 row, view_to_model_col (eti, col), &button);
+				 row, col, &button);
+		g_print("Double click\n");
 		break;
 	}
 	case GDK_MOTION_NOTIFY: {
@@ -1731,11 +1797,13 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 		if (eti->tooltip->timer > 0)
 			gtk_timeout_remove (eti->tooltip->timer);
+#if 0
 		eti->tooltip->col = col;
 		eti->tooltip->row = row;
 		eti->tooltip->cx = e->motion.x;
 		eti->tooltip->cy = e->motion.y;
 		eti->tooltip->timer = gtk_timeout_add (100, (GSourceFunc)_do_tooltip, eti);
+#endif
 
 		if (cursor_row == view_to_model_row(eti, row) && cursor_col == view_to_model_col(eti, col)){
 			ecell_view = eti->cell_views [col];
@@ -1755,6 +1823,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	case GDK_KEY_PRESS: {
 		gint cursor_row, cursor_col;
 		gint handled = TRUE;
+
 		gtk_object_get(GTK_OBJECT(eti->selection),
 			       "cursor_row", &cursor_row,
 			       "cursor_col", &cursor_col,
@@ -1768,6 +1837,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		if (cursor_col == -1)
 			return FALSE;
 
+		eti->in_key_press = TRUE;
+
 		switch (e->key.keyval){
 		case GDK_Left:
 			if (eti_editing (eti)) {
@@ -1775,7 +1846,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				break;
 			}
 			
-			if (cursor_col != view_to_model_col(eti, 0))
+			if (eti->cursor_mode != E_TABLE_CURSOR_LINE && cursor_col != view_to_model_col(eti, 0))
 				eti_cursor_move_left (eti);
 			break;
 			
@@ -1785,10 +1856,15 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				break;
 			}
 
-			if (cursor_col != view_to_model_col(eti, eti->cols - 1))
+			if (eti->cursor_mode != E_TABLE_CURSOR_LINE && cursor_col != view_to_model_col(eti, eti->cols - 1))
 				eti_cursor_move_right (eti);
 			break;
 			
+		case GDK_Up:
+		case GDK_Down:
+			return_val = e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
+			break;
+#if 0
 		case GDK_Up:
 			if (cursor_row != view_to_model_row(eti, 0))
 				eti_cursor_move_up (eti);
@@ -1802,41 +1878,58 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			else
 				return_val = FALSE;
 			break;
-
+#endif
+		case GDK_Home:
+		case GDK_KP_Home:
+			if (eti->cursor_mode != E_TABLE_CURSOR_LINE) {
+				eti_cursor_move (eti, model_to_view_row(eti, cursor_row), 0);
+				return_val = TRUE;
+			} else 
+				return_val = e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
+			break;
+		case GDK_End:
+		case GDK_KP_End:
+			if (eti->cursor_mode != E_TABLE_CURSOR_LINE) {
+				eti_cursor_move (eti, model_to_view_row(eti, cursor_row), eti->cols - 1);
+				return_val = TRUE;
+			} else 
+				return_val = e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
+			break;
 		case GDK_Tab:
 		case GDK_KP_Tab:
 		case GDK_ISO_Left_Tab:
+			if (eti->cursor_mode == E_TABLE_CURSOR_SPREADSHEET) {
+				if ((e->key.state & GDK_SHIFT_MASK) != 0){
+				/* shift tab */
+					if (cursor_col != view_to_model_col(eti, 0))
+						eti_cursor_move_left (eti);
+					else if (cursor_row != view_to_model_row(eti, 0))
+						eti_cursor_move (eti, model_to_view_row(eti, cursor_row) - 1, eti->cols - 1);
+					else
+						return_val = FALSE;
+				} else {
+					if (cursor_col != view_to_model_col (eti, eti->cols - 1))
+						eti_cursor_move_right (eti);
+					else if (cursor_row != view_to_model_row(eti, eti->rows - 1))
+						eti_cursor_move (eti, model_to_view_row(eti, cursor_row) + 1, 0);
+					else 
+						return_val = FALSE;
+				}
+				gtk_object_get(GTK_OBJECT(eti->selection),
+					       "cursor_row", &cursor_row,
+					       "cursor_col", &cursor_col,
+					       NULL);
+
+				if (cursor_col >= 0 && cursor_row >= 0 && return_val &&
+				    (!eti_editing(eti)) && e_table_model_is_cell_editable(eti->selection->model, cursor_col, cursor_row)) {
+					e_table_item_enter_edit (eti, model_to_view_col(eti, cursor_col), model_to_view_row(eti, cursor_row));
+				}
+				break;
+			} else {
 			/* Let tab send you to the next widget. */
 			return_val = FALSE;
 			break;
-#if 0
-			if ((e->key.state & GDK_SHIFT_MASK) != 0){
-				/* shift tab */
-				if (cursor_col != view_to_model_col(eti, 0))
-					eti_cursor_move_left (eti);
-				else if (cursor_row != view_to_model_row(eti, 0))
-					eti_cursor_move (eti, model_to_view_row(eti, cursor_row) - 1, eti->cols - 1);
-				else
-					return_val = FALSE;
-			} else {
-				if (cursor_col != view_to_model_col (eti, eti->cols - 1))
-					eti_cursor_move_right (eti);
-				else if (cursor_row != view_to_model_row(eti, eti->rows - 1))
-					eti_cursor_move (eti, model_to_view_row(eti, cursor_row) + 1, 0);
-				else 
-					return_val = FALSE;
 			}
-			gtk_object_get(GTK_OBJECT(eti->selection),
-				       "cursor_row", &cursor_row,
-				       "cursor_col", &cursor_col,
-				       NULL);
-
-			if (cursor_col >= 0 && cursor_row >= 0 && return_val &&
-			    (!eti_editing(eti)) && e_table_model_is_cell_editable(eti->selection->model, cursor_col, cursor_row)) {
-				e_table_item_enter_edit (eti, model_to_view_col(eti, cursor_col), model_to_view_row(eti, cursor_row));
-			}
-			break;
-#endif
 
 		case GDK_Return:
 		case GDK_KP_Enter:
@@ -1851,9 +1944,10 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
 #endif
 			}
-			return_val = FALSE;
 			gtk_signal_emit (GTK_OBJECT (eti), eti_signals [KEY_PRESS],
 					 model_to_view_row(eti, cursor_row), cursor_col, e, &return_val);
+			if (!return_val)
+				return_val = e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
 			break;
 			
 		default:
@@ -1872,13 +1966,18 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			if (!eti_editing (eti)){
 				gtk_signal_emit (GTK_OBJECT (eti), eti_signals [KEY_PRESS],
 						 model_to_view_row(eti, cursor_row), cursor_col, e, &return_val);
+				if (!return_val)
+					e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
 			} else {
 				ecell_view = eti->cell_views [eti->editing_col];
 				return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
 							       view_to_model_col(eti, eti->editing_col),
 							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
+				if (!return_val)
+					e_table_selection_model_key_press(eti->selection, (GdkEventKey *) e);
 			}
 		}
+		eti->in_key_press = FALSE;
 		break;
 	}
 	
@@ -1909,6 +2008,12 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		eti->tooltip->timer = 0;
 		break;
 
+	case GDK_FOCUS_CHANGE:
+		if (! e->focus_change.in) {
+			if (eti_editing (eti))
+				e_table_item_leave_edit (eti);
+		}
+
 	default:
 		return_val = FALSE;
 	}
@@ -1919,32 +2024,33 @@ static void
 eti_class_init (GtkObjectClass *object_class)
 {
 	GnomeCanvasItemClass *item_class = (GnomeCanvasItemClass *) object_class;
-	ETableItemClass *eti_class = (ETableItemClass *) object_class;
+	ETableItemClass *eti_class  = (ETableItemClass *) object_class;
 	
-	eti_parent_class = gtk_type_class (PARENT_OBJECT_TYPE);
+	eti_parent_class            = gtk_type_class (PARENT_OBJECT_TYPE);
 	
-	object_class->destroy = eti_destroy;
-	object_class->set_arg = eti_set_arg;
-	object_class->get_arg = eti_get_arg;
+	object_class->destroy       = eti_destroy;
+	object_class->set_arg       = eti_set_arg;
+	object_class->get_arg       = eti_get_arg;
 
-	item_class->update      = eti_update;
-	item_class->realize     = eti_realize;
-	item_class->unrealize   = eti_unrealize;
-	item_class->draw        = eti_draw;
-	item_class->point       = eti_point;
-	item_class->event       = eti_event;
+	item_class->update          = eti_update;
+	item_class->realize         = eti_realize;
+	item_class->unrealize       = eti_unrealize;
+	item_class->draw            = eti_draw;
+	item_class->point           = eti_point;
+	item_class->event           = eti_event;
 	
-	eti_class->cursor_change = NULL;
-	eti_class->double_click  = NULL;
-	eti_class->right_click   = NULL;
-	eti_class->click         = NULL;
-	eti_class->key_press     = NULL;
+	eti_class->cursor_change    = NULL;
+	eti_class->cursor_activated = NULL;
+	eti_class->double_click     = NULL;
+	eti_class->right_click      = NULL;
+	eti_class->click            = NULL;
+	eti_class->key_press        = NULL;
 
-	gtk_object_add_arg_type ("ETableItem::ETableHeader", GTK_TYPE_OBJECT,
+	gtk_object_add_arg_type ("ETableItem::ETableHeader", E_TABLE_HEADER_TYPE,
 				 GTK_ARG_WRITABLE, ARG_TABLE_HEADER);
-	gtk_object_add_arg_type ("ETableItem::ETableModel", GTK_TYPE_OBJECT,
+	gtk_object_add_arg_type ("ETableItem::ETableModel", E_TABLE_MODEL_TYPE,
 				 GTK_ARG_WRITABLE, ARG_TABLE_MODEL);
-	gtk_object_add_arg_type ("ETableItem::table_selection_model", GTK_TYPE_OBJECT,
+	gtk_object_add_arg_type ("ETableItem::table_selection_model", E_TABLE_SELECTION_MODEL_TYPE,
 				 GTK_ARG_WRITABLE, ARG_TABLE_SELECTION_MODEL);
 	gtk_object_add_arg_type ("ETableItem::drawgrid", GTK_TYPE_BOOL,
 				 GTK_ARG_WRITABLE, ARG_TABLE_DRAW_GRID);
@@ -1972,6 +2078,14 @@ eti_class_init (GtkObjectClass *object_class)
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 
+	eti_signals [CURSOR_ACTIVATED] =
+		gtk_signal_new ("cursor_activated",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (ETableItemClass, cursor_activated),
+				gtk_marshal_NONE__INT,
+				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+
 	eti_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
@@ -2031,6 +2145,14 @@ e_table_item_get_type (void)
 	return type;
 }
 
+/** 
+ * e_table_item_set_cursor:
+ * @eti: %ETableItem which will have the cursor set.
+ * @col: Column to select.  -1 means the last column.
+ * @row: Row to select.  -1 means the last row.
+ *
+ * This routine sets the cursor of the %ETableItem canvas item.
+ */
 void
 e_table_item_set_cursor    (ETableItem *eti, int col, int row)
 {
@@ -2058,6 +2180,14 @@ e_table_item_focus (ETableItem *eti, int col, int row, GdkModifierType state)
 	}
 }
 
+/** 
+ * e_table_item_get_focused_column:
+ * @eti: %ETableItem which will have the cursor retrieved.
+ *
+ * This routine gets the cursor of the %ETableItem canvas item.
+ *
+ * Returns: The current cursor column.
+ */
 gint
 e_table_item_get_focused_column (ETableItem *eti)
 {	
@@ -2073,13 +2203,21 @@ e_table_item_get_focused_column (ETableItem *eti)
 	return cursor_col;
 }
 
-gboolean
-e_table_item_is_row_selected (ETableItem *eti, int row)
-{
-	g_return_val_if_fail (eti != NULL, FALSE);
-	g_return_val_if_fail (E_IS_TABLE_ITEM (eti), FALSE);
+typedef struct {
+	gint view_row;
+	gint view_col;
+	ETableItem *eti;
+} IntsAndEti;
 
-	return e_table_selection_model_is_row_selected(eti->selection, row);
+static gboolean
+region_timeout (gpointer data)
+{
+	IntsAndEti *iae = data;
+
+	eti_request_region_show (iae->eti, iae->view_col, iae->view_row, iae->view_col, iae->view_row);
+	gtk_object_unref(GTK_OBJECT(iae->eti));
+	g_free(iae);
+	return FALSE;
 }
 
 static void
@@ -2087,13 +2225,24 @@ eti_cursor_change (ETableSelectionModel *selection, int row, int col, ETableItem
 {
 	int view_row = model_to_view_row(eti, row);
 	int view_col = model_to_view_col(eti, col);
+	IntsAndEti *iae;
 
 	if (view_row == -1 || view_col == -1) {
 		e_table_item_leave_edit (eti);
 		return;
 	}
 
-	eti_request_region_show (eti, view_col, view_row, view_col, view_row);
+	if (!eti->in_key_press) {
+	iae = g_new(IntsAndEti, 1);
+	iae->view_row = view_row;
+	iae->view_col = view_col;
+	iae->eti = eti;
+	gtk_object_ref(GTK_OBJECT(eti));
+	g_timeout_add(DOUBLE_CLICK_TIME + 10, region_timeout, iae);
+	} else {
+		eti_request_region_show (eti, view_col, view_row, view_col, view_row);
+	}
+
 	e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti));
 	if (eti_editing(eti))
 		e_table_item_leave_edit (eti);
@@ -2103,6 +2252,23 @@ eti_cursor_change (ETableSelectionModel *selection, int row, int col, ETableItem
 	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(eti));
 }
 
+static void
+eti_cursor_activated (ETableSelectionModel *selection, int row, int col, ETableItem *eti)
+{
+	int view_row = model_to_view_row(eti, row);
+	int view_col = model_to_view_col(eti, col);
+
+	if (view_row == -1 || view_col == -1) {
+		e_table_item_leave_edit (eti);
+		return;
+	}
+
+	if (eti_editing(eti))
+		e_table_item_leave_edit (eti);
+	gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CURSOR_ACTIVATED],
+			 view_row);
+}
+
 static void
 eti_selection_change (ETableSelectionModel *selection, ETableItem *eti)
 {
@@ -2110,6 +2276,16 @@ eti_selection_change (ETableSelectionModel *selection, ETableItem *eti)
 	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(eti));
 }
 
+
+/** 
+ * e_table_item_enter_edit
+ * @eti: %ETableItem which will start being edited
+ * @col: The view col to edit.
+ * @row: The view row to edit.
+ *
+ * This routine starts the given %ETableItem editing at the given view
+ * column and row.
+ */
 void
 e_table_item_enter_edit (ETableItem *eti, int col, int row)
 {
@@ -2125,6 +2301,12 @@ e_table_item_enter_edit (ETableItem *eti, int col, int row)
 	eti->edit_ctx = e_cell_enter_edit (eti->cell_views [col], view_to_model_col(eti, col), col, row);
 }
 
+/** 
+ * e_table_item_leave_edit
+ * @eti: %ETableItem which will stop being edited
+ *
+ * This routine stops the given %ETableItem from editing.
+ */
 void
 e_table_item_leave_edit (ETableItem *eti)
 {
@@ -2150,6 +2332,20 @@ e_table_item_leave_edit (ETableItem *eti)
 			   col, row, edit_ctx);
 }
 
+/** 
+ * e_table_item_compute_location
+ * @eti: %ETableItem to look in.
+ * @x: A pointer to the x location to find in the %ETableItem.
+ * @y: A pointer to the y location to find in the %ETableItem.
+ * @row: A pointer to the location to store the found row in.
+ * @col: A pointer to the location to store the found col in.
+ *
+ * This routine locates the pixel location (*x, *y) in the
+ * %ETableItem.  If that location is in the %ETableItem, *row and *col
+ * are set to the view row and column where it was found.  If that
+ * location is not in the %ETableItem, the height of the %ETableItem
+ * is removed from the value y points to.
+ */
 void 
 e_table_item_compute_location (ETableItem        *eti,
 			       int               *x,
@@ -2448,6 +2644,15 @@ e_table_item_printable_destroy (GtkObject *object,
 	g_free(itemcontext);
 }
 
+/** 
+ * e_table_item_get_printable
+ * @eti: %ETableItem which will be printed
+ *
+ * This routine creates and returns an %EPrintable that can be used to
+ * print the given %ETableItem.
+ *
+ * Returns: The %EPrintable. 
+ */
 EPrintable *
 e_table_item_get_printable (ETableItem *item)
 {
