@@ -3,14 +3,15 @@
  * e-table-selection-model.c: a Table Selection Model
  *
  * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ *   Christopher James Lahey <clahey@ximian.com>
  *
- * (C) 1999 Helix Code, Inc.
+ * (C) 2000, 2001 Ximian, Inc.
  */
 #include <config.h>
 #include <gtk/gtksignal.h>
 #include "e-table-selection-model.h"
 #include "gal/util/e-util.h"
+#include <gdk/gdkkeysyms.h>
 
 #define ETSM_CLASS(e) ((ETableSelectionModelClass *)((GtkObject *)e)->klass)
 
@@ -26,8 +27,11 @@
 
 static GtkObjectClass *e_table_selection_model_parent_class;
 
+static void etsm_select_single_row (ETableSelectionModel *selection, int row);
+
 enum {
 	CURSOR_CHANGED,
+	CURSOR_ACTIVATED,
 	SELECTION_CHANGED,
 	LAST_SIGNAL
 };
@@ -40,6 +44,8 @@ enum {
 	ARG_SORTER,
 	ARG_CURSOR_ROW,
 	ARG_CURSOR_COL,
+	ARG_SELECTION_MODE,
+	ARG_CURSOR_MODE,
 };
 
 static void
@@ -72,6 +78,8 @@ model_row_inserted(ETableModel *etm, int row, ETableSelectionModel *etsm)
 		etsm->selection[box] = (etsm->selection[box] & BITMASK_LEFT(row)) | ((etsm->selection[box] & BITMASK_RIGHT(row)) >> 1);
 		etsm->row_count ++;
 	}
+	if (etsm->cursor_row >= row)
+		etsm->cursor_row ++;
 }
 
 static void
@@ -80,6 +88,7 @@ model_row_deleted(ETableModel *etm, int row, ETableSelectionModel *etsm)
 	int box;
 	int i;
 	int last;
+	int selected = FALSE;
 	if(etsm->row_count >= 0) {
 		guint32 bitmask;
 		box = row >> 5;
@@ -87,6 +96,7 @@ model_row_deleted(ETableModel *etm, int row, ETableSelectionModel *etsm)
 
 		/* Build bitmasks for the left and right half of the box */
 		bitmask = BITMASK_RIGHT(row) >> 1;
+		selected = e_table_selection_model_is_row_selected(etsm, row);
 		/* Shift right half of box one bit to the left. */
 		etsm->selection[box] = (etsm->selection[box] & BITMASK_LEFT(row))| ((etsm->selection[box] & bitmask) << 1);
 
@@ -97,14 +107,20 @@ model_row_deleted(ETableModel *etm, int row, ETableSelectionModel *etsm)
 			for (i = box + 1; i < last; i++) {
 				etsm->selection[i] = (etsm->selection[i] << 1) | (etsm->selection[i + 1] >> 31);
 			}
-			etsm->selection[i] = etsm->selection[i] << 1;
+			/* this over-runs our memory! */
+			/*etsm->selection[i] = etsm->selection[i] << 1; */
 		}
 		etsm->row_count --;
 		/* Remove the last word if not needed. */
 		if ((etsm->row_count & 0x1f) == 0) {
 			etsm->selection = g_renew(gint, etsm->selection, etsm->row_count >> 5);
 		}
+		if (selected && etsm->mode == GTK_SELECTION_SINGLE) {
+			etsm_select_single_row (etsm, row > 0 ? row - 1 : 0);
+		}
 	}
+	if (etsm->cursor_row >= row && etsm->cursor_row > 0)
+		etsm->cursor_row --;
 }
 
 #else
@@ -204,6 +220,14 @@ etsm_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_CURSOR_COL:
 		GTK_VALUE_INT(*arg) = etsm->cursor_col;
 		break;
+
+	case ARG_SELECTION_MODE:
+		GTK_VALUE_ENUM(*arg) = etsm->mode;
+		break;
+
+	case ARG_CURSOR_MODE:
+		GTK_VALUE_ENUM(*arg) = etsm->cursor_mode;
+		break;
 	}
 }
 
@@ -230,6 +254,17 @@ etsm_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_CURSOR_COL:
 		e_table_selection_model_do_something(etsm, etsm->cursor_row, GTK_VALUE_INT(*arg), 0);
 		break;
+
+	case ARG_SELECTION_MODE:
+		etsm->mode = GTK_VALUE_ENUM(*arg);
+		if (etsm->mode == GTK_SELECTION_SINGLE) {
+			e_table_selection_model_do_something(etsm, etsm->cursor_row, etsm->cursor_col, 0);
+		}
+		break;
+
+	case ARG_CURSOR_MODE:
+		etsm->cursor_mode = GTK_VALUE_ENUM(*arg);
+		break;
 	}
 }
 
@@ -242,6 +277,8 @@ e_table_selection_model_init (ETableSelectionModel *selection)
 	selection->selection_start_row = 0;
 	selection->cursor_row = -1;
 	selection->cursor_col = -1;
+	selection->mode = GTK_SELECTION_MULTIPLE;
+	selection->cursor_mode = E_TABLE_CURSOR_SIMPLE;
 }
 
 static void
@@ -265,6 +302,14 @@ e_table_selection_model_class_init (ETableSelectionModelClass *klass)
 				gtk_marshal_NONE__INT_INT,
 				GTK_TYPE_NONE, 2, GTK_TYPE_INT, GTK_TYPE_INT);
 
+	e_table_selection_model_signals [CURSOR_ACTIVATED] =
+		gtk_signal_new ("cursor_activated",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (ETableSelectionModelClass, cursor_activated),
+				gtk_marshal_NONE__INT_INT,
+				GTK_TYPE_NONE, 2, GTK_TYPE_INT, GTK_TYPE_INT);
+
 	e_table_selection_model_signals [SELECTION_CHANGED] =
 		gtk_signal_new ("selection_changed",
 				GTK_RUN_LAST,
@@ -273,7 +318,8 @@ e_table_selection_model_class_init (ETableSelectionModelClass *klass)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
-	klass->cursor_changed = NULL;
+	klass->cursor_changed    = NULL;
+	klass->cursor_activated  = NULL;
 	klass->selection_changed = NULL;
 
 	gtk_object_class_add_signals (object_class, e_table_selection_model_signals, LAST_SIGNAL);
@@ -286,17 +332,37 @@ e_table_selection_model_class_init (ETableSelectionModelClass *klass)
 				 GTK_ARG_READWRITE, ARG_CURSOR_ROW);
 	gtk_object_add_arg_type ("ETableSelectionModel::cursor_col", GTK_TYPE_INT,
 				 GTK_ARG_READWRITE, ARG_CURSOR_COL);
+	gtk_object_add_arg_type ("ETableSelectionModel::selection_mode", GTK_TYPE_ENUM,
+				 GTK_ARG_READWRITE, ARG_SELECTION_MODE);
+	gtk_object_add_arg_type ("ETableSelectionModel::cursor_mode", GTK_TYPE_ENUM,
+				 GTK_ARG_READWRITE, ARG_CURSOR_MODE);
 }
 
 E_MAKE_TYPE(e_table_selection_model, "ETableSelectionModel", ETableSelectionModel,
 	    e_table_selection_model_class_init, e_table_selection_model_init, PARENT_TYPE);
 
+/** 
+ * e_table_selection_model_new
+ *
+ * This routine creates a new #ETableSelectionModel.
+ *
+ * Returns: The new #ETableSelectionModel.
+ */
 ETableSelectionModel *
 e_table_selection_model_new (void)
 {
 	return gtk_type_new (e_table_selection_model_get_type ());
 }
 
+/** 
+ * e_table_selection_model_is_row_selected
+ * @selection: #ETableSelectionModel to check
+ * @n: The row to check
+ *
+ * This routine calculates whether the given row is selected.
+ *
+ * Returns: %TRUE if the given row is selected
+ */
 gboolean
 e_table_selection_model_is_row_selected (ETableSelectionModel *selection,
 					 gint                 n)
@@ -307,6 +373,15 @@ e_table_selection_model_is_row_selected (ETableSelectionModel *selection,
 		return (selection->selection[BOX(n)] >> OFFSET(n)) & 0x1;
 }
 
+/** 
+ * e_table_selection_model_foreach
+ * @selection: #ETableSelectionModel to traverse
+ * @callback: The callback function to call back.
+ * @closure: The closure
+ *
+ * This routine calls the given callback function once for each
+ * selected row, passing closure as the closure.
+ */
 void 
 e_table_selection_model_foreach     (ETableSelectionModel *selection,
 				     ETableForeachFunc callback,
@@ -368,10 +443,96 @@ change_selection(ETableSelectionModel *selection, int start, int end, gboolean g
 	}
 }
 
-void             e_table_selection_model_do_something      (ETableSelectionModel *selection,
-							    guint                 row,
-							    guint                 col,
-							    GdkModifierType       state)
+static void
+etsm_select_single_row (ETableSelectionModel *selection, int row)
+{
+	int i;
+	for (i = 0; i < ((selection->row_count + 31) / 32); i++) {
+		if (!((i == BOX(row) && selection->selection[i] == BITMASK(row)) ||
+		      (i != BOX(row) && selection->selection[i] == 0))) {
+			g_free(selection->selection);
+			selection->selection = g_new0(gint, (selection->row_count + 31) / 32);
+			selection->selection[BOX(row)] = BITMASK(row);
+
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals [SELECTION_CHANGED]);
+			break;
+		}
+	}
+
+	selection->selection_start_row = row;
+}
+
+static void
+etsm_toggle_single_row (ETableSelectionModel *selection, int row)
+{
+	if (selection->selection[BOX(row)] & BITMASK(row))
+		selection->selection[BOX(row)] &= ~BITMASK(row);
+	else
+		selection->selection[BOX(row)] |= BITMASK(row);
+	selection->selection_start_row = row;
+	gtk_signal_emit(GTK_OBJECT(selection),
+			e_table_selection_model_signals [SELECTION_CHANGED]);
+}
+
+static void
+etsm_move_selection_end (ETableSelectionModel *selection, int row)
+{
+	int old_start;
+	int old_end;
+	int new_start;
+	int new_end;
+	if (selection->sorter && e_table_sorter_needs_sorting(selection->sorter)) {
+		old_start = MIN (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
+				 e_table_sorter_model_to_sorted(selection->sorter, selection->cursor_row));
+		old_end = MAX (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
+			       e_table_sorter_model_to_sorted(selection->sorter, selection->cursor_row)) + 1;
+		new_start = MIN (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
+				 e_table_sorter_model_to_sorted(selection->sorter, row));
+		new_end = MAX (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
+			       e_table_sorter_model_to_sorted(selection->sorter, row)) + 1;
+	} else {
+		old_start = MIN (selection->selection_start_row, selection->cursor_row);
+		old_end = MAX (selection->selection_start_row, selection->cursor_row) + 1;
+		new_start = MIN (selection->selection_start_row, row);
+		new_end = MAX (selection->selection_start_row, row) + 1;
+	}
+	/* This wouldn't work nearly so smoothly if one end of the selection weren't held in place. */
+	if (old_start < new_start)
+		change_selection(selection, old_start, new_start, FALSE);
+	if (new_start < old_start)
+		change_selection(selection, new_start, old_start, TRUE);
+	if (old_end < new_end)
+		change_selection(selection, old_end, new_end, TRUE);
+	if (new_end < old_end)
+		change_selection(selection, new_end, old_end, FALSE);
+	gtk_signal_emit(GTK_OBJECT(selection),
+			e_table_selection_model_signals [SELECTION_CHANGED]);
+}
+
+static void
+etsm_set_selection_end (ETableSelectionModel *selection, int row)
+{
+	etsm_select_single_row(selection, selection->selection_start_row);
+	selection->cursor_row = selection->selection_start_row;
+	etsm_move_selection_end(selection, row);
+}
+
+/** 
+ * e_table_selection_model_do_something
+ * @selection: #ETableSelectionModel to do something to.
+ * @row: The row to do something in.
+ * @col: The col to do something in.
+ * @state: The state in which to do something.
+ *
+ * This routine does whatever is appropriate as if the user clicked
+ * the mouse in the given row and column.
+ */
+void
+e_table_selection_model_do_something (ETableSelectionModel *selection,
+				      guint                 row,
+				      guint                 col,
+				      GdkModifierType       state)
 {
 	gint shift_p = state & GDK_SHIFT_MASK;
 	gint ctrl_p = state & GDK_CONTROL_MASK;
@@ -383,61 +544,23 @@ void             e_table_selection_model_do_something      (ETableSelectionModel
 		}
 	}
 	if (selection->row_count >= 0 && row < selection->row_count) {
-		if (shift_p) {
-			int old_start;
-			int old_end;
-			int new_start;
-			int new_end;
-			if (selection->sorter && e_table_sorter_needs_sorting(selection->sorter)) {
-				old_start = MIN (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
-						 e_table_sorter_model_to_sorted(selection->sorter, selection->cursor_row));
-				old_end = MAX (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
-					       e_table_sorter_model_to_sorted(selection->sorter, selection->cursor_row)) + 1;
-				new_start = MIN (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
-						 e_table_sorter_model_to_sorted(selection->sorter, row));
-				new_end = MAX (e_table_sorter_model_to_sorted(selection->sorter, selection->selection_start_row),
-					       e_table_sorter_model_to_sorted(selection->sorter, row)) + 1;
+		switch (selection->mode) {
+		case GTK_SELECTION_SINGLE:
+			etsm_select_single_row (selection, row);
+			break;
+		case GTK_SELECTION_BROWSE:
+		case GTK_SELECTION_MULTIPLE:
+		case GTK_SELECTION_EXTENDED:
+			if (shift_p) {
+				etsm_set_selection_end (selection, row);
 			} else {
-				old_start = MIN (selection->selection_start_row, selection->cursor_row);
-				old_end = MAX (selection->selection_start_row, selection->cursor_row) + 1;
-				new_start = MIN (selection->selection_start_row, row);
-				new_end = MAX (selection->selection_start_row, row) + 1;
-			}
-			/* This wouldn't work nearly so smoothly if one end of the selection held in place. */
-			if (old_start < new_start)
-				change_selection(selection, old_start, new_start, FALSE);
-			if (new_start < old_start)
-				change_selection(selection, new_start, old_start, TRUE);
-			if (old_end < new_end)
-				change_selection(selection, old_end, new_end, TRUE);
-			if (new_end < old_end)
-				change_selection(selection, new_end, old_end, FALSE);
-			gtk_signal_emit(GTK_OBJECT(selection),
-					e_table_selection_model_signals [SELECTION_CHANGED]);
-		} else {
-			if (ctrl_p) {
-				if (selection->selection[BOX(row)] & BITMASK(row))
-					selection->selection[BOX(row)] &= ~BITMASK(row);
-				else
-					selection->selection[BOX(row)] |= BITMASK(row);
-				gtk_signal_emit(GTK_OBJECT(selection),
-						e_table_selection_model_signals [SELECTION_CHANGED]);
-			} else {
-				int i;
-				for (i = 0; i < ((selection->row_count + 31) / 32); i++) {
-					if (!((i == BOX(row) && selection->selection[i] == BITMASK(row)) ||
-					      (i != BOX(row) && selection->selection[i] == 0))) {
-						g_free(selection->selection);
-						selection->selection = g_new0(gint, (selection->row_count + 31) / 32);
-						selection->selection[BOX(row)] = BITMASK(row);
-
-						gtk_signal_emit(GTK_OBJECT(selection),
-								e_table_selection_model_signals [SELECTION_CHANGED]);
-						break;
-					}
+				if (ctrl_p) {
+					etsm_toggle_single_row (selection, row);
+				} else {
+					etsm_select_single_row (selection, row);
 				}
 			}
-			selection->selection_start_row = row;
+			break;
 		}
 		if (selection->cursor_row != row ||
 		    selection->cursor_col != col) {
@@ -445,14 +568,30 @@ void             e_table_selection_model_do_something      (ETableSelectionModel
 			selection->cursor_col = col;
 			gtk_signal_emit(GTK_OBJECT(selection),
 					e_table_selection_model_signals[CURSOR_CHANGED], row, col);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], row, col);
 		}
 	}
 }
 
-void             e_table_selection_model_maybe_do_something      (ETableSelectionModel *selection,
-								  guint                 row,
-								  guint                 col,
-								  GdkModifierType       state)
+/** 
+ * e_table_selection_model_maybe_do_something
+ * @selection: #ETableSelectionModel to do something to.
+ * @row: The row to do something in.
+ * @col: The col to do something in.
+ * @state: The state in which to do something.
+ *
+ * If this row is selected, this routine just moves the cursor row and
+ * column.  Otherwise, it does the same thing as
+ * e_table_selection_model_do_something().  This is for being used on
+ * right clicks and other events where if the user hit the selection,
+ * they don't want it to change.
+ */
+void
+e_table_selection_model_maybe_do_something      (ETableSelectionModel *selection,
+						 guint                 row,
+						 guint                 col,
+						 GdkModifierType       state)
 {
 	if (e_table_selection_model_is_row_selected(selection, row)) {
 		selection->cursor_row = row;
@@ -462,6 +601,136 @@ void             e_table_selection_model_maybe_do_something      (ETableSelectio
 	}
 }
 
+static gint
+move_selection (ETableSelectionModel *selection,
+		gboolean              up,
+		GdkModifierType       state)
+{
+	int row = selection->cursor_row;
+	int col = selection->cursor_col;
+	int cursor_activated = TRUE;
+
+	gint shift_p = state & GDK_SHIFT_MASK;
+	gint ctrl_p = state & GDK_CONTROL_MASK;
+
+	row = e_table_sorter_model_to_sorted(selection->sorter, row);
+	if (up)
+		row--;
+	else
+		row++;
+	if (row < 0)
+		row = 0;
+	if (row >= selection->row_count)
+		row = selection->row_count - 1;
+	row = e_table_sorter_sorted_to_model(selection->sorter, row);
+
+	switch (selection->mode) {
+	case GTK_SELECTION_BROWSE:
+		if (shift_p) {
+			etsm_set_selection_end (selection, row);
+		} else if (!ctrl_p) {
+			etsm_select_single_row (selection, row);
+		} else
+			cursor_activated = FALSE;
+		break;
+	case GTK_SELECTION_SINGLE:
+	case GTK_SELECTION_MULTIPLE:
+	case GTK_SELECTION_EXTENDED:
+		etsm_select_single_row (selection, row);
+		break;
+	}
+	if (row != -1) {
+		selection->cursor_row = row;
+		gtk_signal_emit(GTK_OBJECT(selection),
+				e_table_selection_model_signals[CURSOR_CHANGED], row, col);
+		if (cursor_activated)
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], row, col);
+	}
+	return TRUE;
+}
+
+/** 
+ * e_table_selection_model_key_press
+ * @selection: #ETableSelectionModel to affect.
+ * @key: The event.
+ *
+ * This routine does whatever is appropriate as if the user pressed
+ * the given key.
+ *
+ * Returns: %TRUE if the #ETableSelectionModel used the key.
+ */
+gint
+e_table_selection_model_key_press      (ETableSelectionModel *selection,
+					GdkEventKey          *key)
+{
+	switch (key->keyval) {
+	case GDK_Up:
+		return move_selection(selection, TRUE, key->state);
+		break;
+	case GDK_Down:
+		return move_selection(selection, FALSE, key->state);
+		break;
+	case GDK_space:
+	case GDK_KP_Space:
+		if (selection->mode != GTK_SELECTION_SINGLE) {
+			etsm_toggle_single_row (selection, selection->cursor_row);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], selection->cursor_row, selection->cursor_col);
+			return TRUE;
+		}
+		break;
+	case GDK_Return:
+	case GDK_KP_Enter:
+		if (selection->mode != GTK_SELECTION_SINGLE) {
+			etsm_select_single_row (selection, selection->cursor_row);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], selection->cursor_row, selection->cursor_col);
+			return TRUE;
+		}
+		break;
+	case GDK_Home:
+	case GDK_KP_Home:
+		if (selection->cursor_mode == E_TABLE_CURSOR_LINE) {
+			int row = 0;
+
+			row = e_table_sorter_sorted_to_model(selection->sorter, row);
+			selection->cursor_row = row;
+
+			etsm_select_single_row (selection, selection->cursor_row);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_CHANGED], selection->cursor_row, selection->cursor_col);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], selection->cursor_row, selection->cursor_col);
+			return TRUE;
+		}
+		break;
+	case GDK_End:
+	case GDK_KP_End:
+		if (selection->cursor_mode == E_TABLE_CURSOR_LINE) {
+			int row = selection->row_count - 1;
+
+			row = e_table_sorter_sorted_to_model(selection->sorter, row);
+			selection->cursor_row = row;
+
+			etsm_select_single_row (selection, selection->cursor_row);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_CHANGED], selection->cursor_row, selection->cursor_col);
+			gtk_signal_emit(GTK_OBJECT(selection),
+					e_table_selection_model_signals[CURSOR_ACTIVATED], selection->cursor_row, selection->cursor_col);
+			return TRUE;
+		}
+		break;
+	}
+	return FALSE;
+}
+
+/** 
+ * e_table_selection_model_clear
+ * @selection: #ETableSelectionModel to clear
+ *
+ * This routine clears the selection to no rows selected.
+ */
 void
 e_table_selection_model_clear(ETableSelectionModel *selection)
 {
@@ -479,6 +748,14 @@ e_table_selection_model_clear(ETableSelectionModel *selection)
 #define PART(x,n) (((x) & (0x01010101 << n)) >> n)
 #define SECTION(x, n) (((x) >> (n * 8)) & 0xff)
 
+/** 
+ * e_table_selection_model_selected_count
+ * @selection: #ETableSelectionModel to count
+ *
+ * This routine calculates the number of rows selected.
+ *
+ * Returns: The number of rows selected in the given model.
+ */
 gint
 e_table_selection_model_selected_count (ETableSelectionModel *selection)
 {
@@ -505,6 +782,13 @@ e_table_selection_model_selected_count (ETableSelectionModel *selection)
 	return count;
 }
 
+/** 
+ * e_table_selection_model_select_all
+ * @selection: #ETableSelectionModel to select all
+ *
+ * This routine selects all the rows in the given
+ * #ETableSelectionModel.
+ */
 void
 e_table_selection_model_select_all (ETableSelectionModel *selection)
 {
@@ -546,6 +830,13 @@ e_table_selection_model_select_all (ETableSelectionModel *selection)
 			 e_table_selection_model_signals [SELECTION_CHANGED]);
 }
 
+/** 
+ * e_table_selection_model_invert_selection
+ * @selection: #ETableSelectionModel to invert
+ *
+ * This routine inverts all the rows in the given
+ * #ETableSelectionModel.
+ */
 void
 e_table_selection_model_invert_selection (ETableSelectionModel *selection)
 {
