@@ -5,7 +5,7 @@
  * Author:
  *   Miguel de Icaza (miguel@gnu.org)
  *
- * (C) 1999 Helix Code, Inc.
+ * (C) 1999 Ximian, Inc.
  */
 #include <config.h>
 #include <stdlib.h>
@@ -182,10 +182,10 @@ ets_sort_info_changed (ETableSortInfo *info, ETableSorter *ets)
 }
 
 static ETableSorter *ets_closure;
-void **vals_closure;
-int cols_closure;
-int *ascending_closure;
-GCompareFunc *compare_closure;
+static void **vals_closure;
+static int cols_closure;
+static int *ascending_closure;
+static GCompareFunc *compare_closure;
 
 /* FIXME: Make it not cache the second and later columns (as if anyone cares.) */
 
@@ -243,32 +243,6 @@ struct _subinfo {
 	GArray *rowsort;	/* an array of row info's */
 };
 
-static int
-qsort_callback_complex(const void *data1, const void *data2)
-{
-	gint row1 = ((struct _rowinfo *)data1)->row;
-	gint row2 = ((struct _rowinfo *)data2)->row;
-	int j;
-	int sort_count = e_table_sort_info_sorting_get_count(ets_closure->sort_info);
-	int comp_val = 0;
-	int ascending = 1;
-	for (j = 0; j < sort_count; j++) {
-		comp_val = (*(compare_closure[j]))(vals_closure[cols_closure * row1 + j], vals_closure[cols_closure * row2 + j]);
-		ascending = ascending_closure[j];
-		if (comp_val != 0)
-			break;
-	}
-	if (comp_val == 0) {
-		if (row1 < row2)
-			comp_val = -1;
-		if (row1 > row2)
-			comp_val = 1;
-	}
-	if (!ascending)
-		comp_val = -comp_val;
-	return comp_val;
-}
-
 /* builds the info needed to sort everything */
 static struct _subinfo *
 ets_sort_build_subset(ETableSorter *ets, struct _group_info *groupinfo, int start, int *end)
@@ -340,7 +314,7 @@ ets_sort_subset(ETableSorter *ets, struct _subinfo *subinfo, int startoffset)
 	d(printf("sorting subset start %d rows %d\n", startoffset, rowsort->len));
 
 	/* first, sort the actual data */
-	qsort(rowsort->data, rowsort->len, sizeof(struct _rowinfo), qsort_callback_complex);
+	qsort(rowsort->data, rowsort->len, sizeof(struct _rowinfo), qsort_callback);
 
 	/* then put it back in the map table, where appropriate */
 	offset = startoffset;
@@ -401,7 +375,7 @@ ets_sort_by_group (ETableSorter *ets)
 		return;
 
 	/* get all the rows' sort groups */
-	groups = g_malloc(sizeof(struct _group_info) * rows);
+	groups = g_new(struct _group_info, rows);
 	for (i=0;i<rows;i++) {
 		groups[i].row = i;
 		groups[i].group = g_strdup(e_table_model_row_sort_group(ets->source, groups[i].row));
@@ -437,8 +411,6 @@ ets_sort(ETableSorter *ets)
 	if (ets->sorted)
 		return;
 
-	ets->needs_sorting = 0;
-
 	rows = e_table_model_row_count(ets->source);
 	group_cols = e_table_sort_info_grouping_get_count(ets->sort_info);
 	cols = e_table_sort_info_sorting_get_count(ets->sort_info) + group_cols;
@@ -463,10 +435,9 @@ ets_sort(ETableSorter *ets)
 		else
 			column = e_table_sort_info_sorting_get_nth(ets->sort_info, j - group_cols);
 
-		if (column.column > e_table_header_count (ets->full_header))
+		col = e_table_header_get_column_by_col_idx(ets->full_header, column.column);
+		if (col == NULL)
 			col = e_table_header_get_column (ets->full_header, e_table_header_count (ets->full_header) - 1);
-		else
-			col = e_table_header_get_column (ets->full_header, column.column);
 
 		for (i = 0; i < rows; i++) {
 			vals_closure[i * cols + j] = e_table_model_value_at (ets->source, col->col_idx, i);
