@@ -2,10 +2,11 @@
 /*
  * E-table-state.c: Savable state of a table.
  *
- * Author:
- *   Chris Lahey <clahey@helixcode.com>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza (miguel@ximian.com)
  *
- * (C) 2000 Helix Code, Inc.
+ * (C) 2000 Ximian, Inc.
  */
 #include <config.h>
 #include <stdlib.h>
@@ -18,6 +19,8 @@
 
 #define PARENT_TYPE (gtk_object_get_type())
 
+#define STATE_VERSION 0.1
+
 static GtkObjectClass *etst_parent_class;
 
 static void
@@ -39,7 +42,15 @@ etst_class_init (GtkObjectClass *klass)
 	klass->destroy = etst_destroy;
 }
 
-E_MAKE_TYPE(e_table_state, "ETableState", ETableState, etst_class_init, NULL, PARENT_TYPE);
+static void
+etst_init (ETableState *state)
+{
+	state->columns = NULL;
+	state->expansions = NULL;
+	state->sort_info = e_table_sort_info_new();
+}
+
+E_MAKE_TYPE(e_table_state, "ETableState", ETableState, etst_class_init, etst_init, PARENT_TYPE);
 
 ETableState *
 e_table_state_new (void)
@@ -77,35 +88,59 @@ e_table_state_load_from_string  (ETableState *state,
 	}
 }
 
+typedef struct {
+	int column;
+	double expansion;
+} int_and_double;
+
 void
-e_table_state_load_from_node    (ETableState *state,
-				 const xmlNode       *node)
+e_table_state_load_from_node (ETableState *state,
+			      const xmlNode *node)
 {
 	xmlNode *children;
 	GList *list = NULL, *iterator;
+	gdouble state_version;
 	int i;
 
+	state_version = e_xml_get_double_prop_by_name_with_default (
+		node, "state-version", STATE_VERSION);
+
 	if (state->sort_info)
-		gtk_object_unref(GTK_OBJECT(state->sort_info));
+		gtk_object_unref (GTK_OBJECT(state->sort_info));
+
 	state->sort_info = NULL;
-	for (children = node->xmlChildrenNode; children; children = children->next) {
-		if (!strcmp(children->name, "column")) {
-			int *column = g_new(int, 1);
+	children = node->xmlChildrenNode;
+	for (; children; children = children->next) {
+		if (!strcmp (children->name, "column")) {
+			int_and_double *column_info = g_new(int_and_double, 1);
 
-			*column = e_xml_get_integer_prop_by_name(children, "source");
+			column_info->column = e_xml_get_integer_prop_by_name(
+				children, "source");
+			column_info->expansion =
+				e_xml_get_double_prop_by_name_with_default(
+					children, "expansion", -2);
 
-			list = g_list_append(list, column);
-		} else if (state->sort_info == NULL && !strcmp(children->name, "grouping")) {
+			list = g_list_append (list, column_info);
+		} else if (state->sort_info == NULL &&
+			   !strcmp (children->name, "grouping")) {
 			state->sort_info = e_table_sort_info_new();
-			e_table_sort_info_load_from_node(state->sort_info, children);
+			e_table_sort_info_load_from_node(
+				state->sort_info, children, state_version);
 		}
 	}
 	g_free(state->columns);
+	g_free(state->expansions);
 	state->col_count = g_list_length(list);
 	state->columns = g_new(int, state->col_count);
-	for (iterator = list, i = 0; iterator; iterator = g_list_next(iterator), i++) {
-		state->columns[i] = *(int *)iterator->data;
-		g_free(iterator->data);
+	state->expansions = g_new(double, state->col_count);
+
+	for (iterator = list, i = 0; iterator; i++) {
+		int_and_double *column_info = iterator->data;
+		
+		state->columns [i] = column_info->column;
+		state->expansions [i] = column_info->expansion;
+		g_free (column_info);
+		iterator = g_list_next (iterator);
 	}
 	g_list_free(list);
 }
@@ -129,7 +164,7 @@ e_table_state_save_to_string    (ETableState *state)
 	int length;
 	xmlDoc *doc;
 
-	doc = xmlNewDoc(NULL);
+	doc = xmlNewDoc("1.0");
 	xmlDocSetRootElement(doc, e_table_state_save_to_node(state, NULL));
 	xmlDocDumpMemory(doc, &string, &length);
 	xmlFreeDoc(doc);
@@ -151,14 +186,17 @@ e_table_state_save_to_node      (ETableState *state,
 	else
 		node = xmlNewNode (NULL, "ETableState");
 
-	e_xml_set_double_prop_by_name(node, "state-version", 0.0);
+	e_xml_set_double_prop_by_name(node, "state-version", STATE_VERSION);
 
 	for (i = 0; i < state->col_count; i++) {
 		int column = state->columns[i];
+		double expansion = state->expansions[i];
 		xmlNode *new_node;
 
 		new_node = xmlNewChild(node, NULL, "column", NULL);
 		e_xml_set_integer_prop_by_name (new_node, "source", column);
+		if (expansion >= -1)
+			e_xml_set_double_prop_by_name(new_node, "expansion", expansion);
 	}
 
 
@@ -166,3 +204,29 @@ e_table_state_save_to_node      (ETableState *state,
 
 	return node;
 }
+
+/**
+ * e_table_state_duplicate:
+ * @state: The ETableState to duplicate
+ *
+ * This creates a copy of the %ETableState @state
+ *
+ * Returns: The duplicated %ETableState.
+ */
+ETableState *
+e_table_state_duplicate (ETableState *state)
+{
+	ETableState *new_state;
+	char *copy;
+	
+	g_return_val_if_fail (state != NULL, NULL);
+	g_return_val_if_fail (E_IS_TABLE_STATE (state), NULL);
+
+	new_state = e_table_state_new ();
+	copy = e_table_state_save_to_string (state);
+	e_table_state_load_from_string (new_state, copy);
+	g_free (copy);
+
+	return new_state;
+}
+
