@@ -3,19 +3,17 @@
  * E-table.c: A graphical view of a Table.
  *
  * Author:
- *   Miguel de Icaza (miguel@helixcode.com)
- *   Chris Lahey (clahey@helixcode.com)
+ *   Miguel de Icaza (miguel@ximian.com)
+ *   Chris Lahey <clahey@ximian.com>
  *
- * Copyright 1999, Helix Code, Inc
+ * Copyright 1999, 2000, 2001, Ximian, Inc
  */
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
 #include <stdio.h>
+#include "gal/util/e-i18n.h"
 #include <libgnomeui/gnome-canvas.h>
 #include <gtk/gtksignal.h>
 
@@ -42,6 +40,7 @@ static GtkObjectClass *e_table_parent_class;
 
 enum {
 	CURSOR_CHANGE,
+	CURSOR_ACTIVATED,
 	SELECTION_CHANGE,
 	DOUBLE_CLICK,
 	RIGHT_CLICK,
@@ -63,7 +62,6 @@ enum {
 
 enum {
 	ARG_0,
-	ARG_TABLE_DRAW_FOCUS,
 	ARG_LENGTH_THRESHOLD,
 };
 
@@ -72,6 +70,8 @@ static gint et_signals [LAST_SIGNAL] = { 0, };
 static void e_table_fill_table (ETable *e_table, ETableModel *model);
 static gboolean changed_idle (gpointer data);
 
+static void et_grab_focus (GtkWidget *widget);
+
 static void et_drag_begin (GtkWidget *widget,
 			   GdkDragContext *context,
 			   ETable *et);
@@ -116,11 +116,14 @@ static gint e_table_drag_source_event_cb (GtkWidget      *widget,
 					  GdkEvent       *event,
 					  ETable         *table);
 
+static gint et_focus (GtkContainer *container, GtkDirectionType direction);
+
+
 static void
 et_destroy (GtkObject *object)
 {
 	ETable *et = E_TABLE (object);
-	
+
 
 	gtk_signal_disconnect (GTK_OBJECT (et->model),
 			       et->table_model_change_id);
@@ -158,9 +161,9 @@ et_destroy (GtkObject *object)
 		g_source_remove (et->rebuild_idle_id);
 		et->rebuild_idle_id = 0;
 	}
-	
+
 	g_free(et->click_to_add_message);
-	
+
 	(*e_table_parent_class->destroy)(object);
 }
 
@@ -170,8 +173,10 @@ e_table_init (GtkObject *object)
 	ETable *e_table = E_TABLE (object);
 	GtkTable *gtk_table = GTK_TABLE (object);
 
+	GTK_WIDGET_SET_FLAGS (e_table, GTK_CAN_FOCUS);
+
 	gtk_table->homogeneous = FALSE;
-	
+
 	e_table->sort_info = NULL;
 	e_table->group_info_change_id = 0;
 	e_table->reflow_idle_id = 0;
@@ -180,10 +185,10 @@ e_table_init (GtkObject *object)
 	e_table->draw_focus = 1;
 	e_table->cursor_mode = E_TABLE_CURSOR_SIMPLE;
 	e_table->length_threshold = 200;
-	
+
 	e_table->need_rebuild = 0;
 	e_table->rebuild_idle_id = 0;
-	
+
 	e_table->click_to_add_message = NULL;
 
 	e_table->drag_get_data_row = -1;
@@ -200,6 +205,33 @@ e_table_init (GtkObject *object)
 	e_table->spec = NULL;
 }
 
+/* Grab_focus handler for the ETable */
+static void
+et_grab_focus (GtkWidget *widget)
+{
+	ETable *e_table;
+
+	e_table = E_TABLE (widget);
+
+	gtk_widget_grab_focus (GTK_WIDGET (e_table->table_canvas));
+}
+
+/* Focus handler for the ETable */
+static gint
+et_focus (GtkContainer *container, GtkDirectionType direction)
+{
+	ETable *e_table;
+
+	e_table = E_TABLE (container);
+
+	if (container->focus_child) {
+		gtk_container_set_focus_child (container, NULL);
+		return FALSE;
+	}
+
+	return gtk_container_focus (GTK_CONTAINER (e_table->table_canvas), direction);
+}
+
 static void
 header_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc, ETable *e_table)
 {
@@ -210,11 +242,11 @@ header_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc, ETable *e_
 
 	/* When the header item is created ->height == 0,
 	   as the font is only created when everything is realized.
-	   So we set the usize here as well, so that the size of the 
+	   So we set the usize here as well, so that the size of the
 	   header is correct */
 	if (GTK_WIDGET (e_table->header_canvas)->allocation.height !=
 	    E_TABLE_HEADER_ITEM (e_table->header_item)->height)
-		gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1, 
+		gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1,
 				      E_TABLE_HEADER_ITEM (e_table->header_item)->height);
 }
 
@@ -229,25 +261,31 @@ sort_info_changed (ETableSortInfo *info, ETable *et)
 static void
 e_table_setup_header (ETable *e_table)
 {
+	char *pointer;
 	e_table->header_canvas = GNOME_CANVAS (e_canvas_new ());
-	
+	GTK_WIDGET_UNSET_FLAGS (e_table->header_canvas, GTK_CAN_FOCUS);
+
 	gtk_widget_show (GTK_WIDGET (e_table->header_canvas));
 
+	pointer = g_strdup_printf("%p", e_table);
+
 	e_table->header_item = gnome_canvas_item_new (
 		gnome_canvas_root (e_table->header_canvas),
 		e_table_header_item_get_type (),
 		"ETableHeader", e_table->header,
 		"full_header", e_table->full_header,
 		"sort_info", e_table->sort_info,
-		"dnd_code", "(unset)",
+		"dnd_code", pointer,
 		"table", e_table,
 		NULL);
 
+	g_free(pointer);
+
 	gtk_signal_connect (
 		GTK_OBJECT (e_table->header_canvas), "size_allocate",
 		GTK_SIGNAL_FUNC (header_canvas_size_allocate), e_table);
 
-	gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1, 
+	gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1,
 			      E_TABLE_HEADER_ITEM (e_table->header_item)->height);
 }
 
@@ -337,6 +375,14 @@ group_cursor_change (ETableGroup *etg, int row, ETable *et)
 			 row);
 }
 
+static void
+group_cursor_activated (ETableGroup *etg, int row, ETable *et)
+{
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [CURSOR_ACTIVATED],
+			 row);
+}
+
 static void
 group_double_click (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et)
 {
@@ -373,12 +419,13 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 	GdkEventButton click;
 	switch (key->keyval) {
 	case GDK_Page_Down:
-		gtk_adjustment_set_value(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-					 CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value +
-					       (gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20),
-					       0, 
-					       gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->upper - 
-					       gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size));
+		gtk_adjustment_set_value(
+			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
+			CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value +
+			      (gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20),
+			      0,
+			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->upper -
+			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size));
 		click.type = GDK_BUTTON_PRESS;
 		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
 		click.send_event = key->send_event;
@@ -392,9 +439,10 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
-		gtk_adjustment_set_value(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-					 gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value -
-					 (gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20));
+		gtk_adjustment_set_value(
+			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
+			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value -
+			(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20));
 		click.type = GDK_BUTTON_PRESS;
 		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
 		click.send_event = key->send_event;
@@ -439,6 +487,8 @@ changed_idle (gpointer data)
 				      NULL);
 		gtk_signal_connect (GTK_OBJECT (et->group), "cursor_change",
 				    GTK_SIGNAL_FUNC (group_cursor_change), et);
+		gtk_signal_connect (GTK_OBJECT (et->group), "cursor_activated",
+				    GTK_SIGNAL_FUNC (group_cursor_activated), et);
 		gtk_signal_connect (GTK_OBJECT (et->group), "double_click",
 				    GTK_SIGNAL_FUNC (group_double_click), et);
 		gtk_signal_connect (GTK_OBJECT (et->group), "right_click",
@@ -448,7 +498,7 @@ changed_idle (gpointer data)
 		gtk_signal_connect (GTK_OBJECT (et->group), "key_press",
 				    GTK_SIGNAL_FUNC (group_key_press), et);
 		e_table_fill_table (et, et->model);
-		
+
 		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
 				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
 				NULL);
@@ -512,9 +562,22 @@ et_table_row_deleted (ETableModel *table_model, int row, ETable *et)
 static void
 et_canvas_realize (GtkWidget *canvas, ETable *e_table)
 {
-	gnome_canvas_item_set(e_table->white_item,
-			      "fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
-			      NULL);
+	gnome_canvas_item_set(
+		e_table->white_item,
+		"fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
+		NULL);
+}
+
+static void
+et_canvas_button_press (GtkWidget *canvas, GdkEvent *event, ETable *e_table)
+{
+	if (GTK_WIDGET_HAS_FOCUS(canvas)) {
+		GnomeCanvasItem *item = GNOME_CANVAS(canvas)->focused_item;
+
+		if (E_IS_TABLE_ITEM(item)) {
+			e_table_item_leave_edit(E_TABLE_ITEM(item));
+		}
+	}
 }
 
 static void
@@ -556,39 +619,50 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 	gtk_signal_connect (
 		GTK_OBJECT (e_table), "drag_data_received",
 		GTK_SIGNAL_FUNC (et_drag_data_received), e_table);
-	
+
 	gtk_signal_connect (GTK_OBJECT(e_table->table_canvas), "reflow",
 			    GTK_SIGNAL_FUNC (table_canvas_reflow), e_table);
-				 
+
 	gtk_widget_show (GTK_WIDGET (e_table->table_canvas));
 
 
-	e_table->white_item = gnome_canvas_item_new(gnome_canvas_root(e_table->table_canvas),
-						    gnome_canvas_rect_get_type(),
-						    "x1", (double) 0,
-						    "y1", (double) 0,
-						    "x2", (double) 100,
-						    "y2", (double) 100,
-						    "fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
-						    NULL);
-	gtk_signal_connect(GTK_OBJECT(e_table->table_canvas), "realize",
-			   GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
-	e_table->canvas_vbox = gnome_canvas_item_new(gnome_canvas_root(e_table->table_canvas),
-						     e_canvas_vbox_get_type(),
-						     "spacing", 10.0,
-						     NULL);
+	e_table->white_item = gnome_canvas_item_new(
+		gnome_canvas_root(e_table->table_canvas),
+		gnome_canvas_rect_get_type(),
+		"x1", (double) 0,
+		"y1", (double) 0,
+		"x2", (double) 100,
+		"y2", (double) 100,
+		"fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
+		NULL);
+
+	gtk_signal_connect (
+		GTK_OBJECT(e_table->table_canvas), "realize",
+		GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
+	gtk_signal_connect (
+		GTK_OBJECT(e_table->table_canvas), "button_press_event",
+		GTK_SIGNAL_FUNC(et_canvas_button_press), e_table);
+	e_table->canvas_vbox = gnome_canvas_item_new(
+		gnome_canvas_root(e_table->table_canvas),
+		e_canvas_vbox_get_type(),
+		"spacing", 10.0,
+		NULL);
 
 	if (e_table->use_click_to_add) {
-		e_table->click_to_add = gnome_canvas_item_new (GNOME_CANVAS_GROUP(e_table->canvas_vbox),
-							       e_table_click_to_add_get_type (),
-							       "header", e_table->header,
-							       "model", e_table->model,
-							       "message", e_table->click_to_add_message,
-							       NULL);
-		
-		e_canvas_vbox_add_item(E_CANVAS_VBOX(e_table->canvas_vbox), e_table->click_to_add);
-		gtk_signal_connect(GTK_OBJECT (e_table->click_to_add), "cursor_change",
-				   GTK_SIGNAL_FUNC(click_to_add_cursor_change), e_table);
+		e_table->click_to_add = gnome_canvas_item_new (
+			GNOME_CANVAS_GROUP(e_table->canvas_vbox),
+			e_table_click_to_add_get_type (),
+			"header", e_table->header,
+			"model", e_table->model,
+			"message", e_table->click_to_add_message,
+			NULL);
+
+		e_canvas_vbox_add_item (
+			E_CANVAS_VBOX(e_table->canvas_vbox),
+			e_table->click_to_add);
+		gtk_signal_connect (
+			GTK_OBJECT (e_table->click_to_add), "cursor_change",
+			GTK_SIGNAL_FUNC(click_to_add_cursor_change), e_table);
 	}
 
 	e_table->group = e_table_group_new (
@@ -597,16 +671,19 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 		model, e_table->sort_info, 0);
 	e_canvas_vbox_add_item(E_CANVAS_VBOX(e_table->canvas_vbox), GNOME_CANVAS_ITEM(e_table->group));
 
-	gnome_canvas_item_set(GNOME_CANVAS_ITEM(e_table->group),
-			      "drawgrid", e_table->draw_grid,
-			      "drawfocus", e_table->draw_focus,
-			      "cursor_mode", e_table->cursor_mode,
-			      "length_threshold", e_table->length_threshold,
-			      "table_selection_model", e_table->selection,
-			      NULL);
+	gnome_canvas_item_set(
+		GNOME_CANVAS_ITEM(e_table->group),
+		"drawgrid", e_table->draw_grid,
+		"drawfocus", e_table->draw_focus,
+		"cursor_mode", e_table->cursor_mode,
+		"length_threshold", e_table->length_threshold,
+		"table_selection_model", e_table->selection,
+		NULL);
 
 	gtk_signal_connect (GTK_OBJECT (e_table->group), "cursor_change",
 			    GTK_SIGNAL_FUNC(group_cursor_change), e_table);
+	gtk_signal_connect (GTK_OBJECT (e_table->group), "cursor_activated",
+			    GTK_SIGNAL_FUNC(group_cursor_activated), e_table);
 	gtk_signal_connect (GTK_OBJECT (e_table->group), "double_click",
 			    GTK_SIGNAL_FUNC(group_double_click), e_table);
 	gtk_signal_connect (GTK_OBJECT (e_table->group), "right_click",
@@ -615,7 +692,7 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 			    GTK_SIGNAL_FUNC(group_click), e_table);
 	gtk_signal_connect (GTK_OBJECT (e_table->group), "key_press",
 			    GTK_SIGNAL_FUNC(group_key_press), e_table);
-	
+
 	e_table->table_model_change_id = gtk_signal_connect (
 		GTK_OBJECT (model), "model_changed",
 		GTK_SIGNAL_FUNC (et_table_model_changed), e_table);
@@ -627,11 +704,11 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 	e_table->table_cell_change_id = gtk_signal_connect (
 		GTK_OBJECT (model), "model_cell_changed",
 		GTK_SIGNAL_FUNC (et_table_cell_changed), e_table);
-	
+
 	e_table->table_row_inserted_id = gtk_signal_connect (
 		GTK_OBJECT (model), "model_row_inserted",
 		GTK_SIGNAL_FUNC (et_table_row_inserted), e_table);
-	
+
 	e_table->table_row_deleted_id = gtk_signal_connect (
 		GTK_OBJECT (model), "model_row_deleted",
 		GTK_SIGNAL_FUNC (et_table_row_deleted), e_table);
@@ -645,7 +722,9 @@ e_table_fill_table (ETable *e_table, ETableModel *model)
 }
 
 static ETableCol *
-et_col_spec_to_col (ETable *e_table, ETableColumnSpecification *col_spec, ETableExtras *ete)
+et_col_spec_to_col (ETable                    *e_table,
+		    ETableColumnSpecification *col_spec,
+		    ETableExtras              *ete)
 {
 	ETableCol *col = NULL;
 	ECell *cell;
@@ -657,25 +736,31 @@ et_col_spec_to_col (ETable *e_table, ETableColumnSpecification *col_spec, ETable
 	if (cell && compare) {
 		if (col_spec->pixbuf && *col_spec->pixbuf) {
 			GdkPixbuf *pixbuf;
-			
-			pixbuf = e_table_extras_get_pixbuf(ete, col_spec->pixbuf);
+
+			pixbuf = e_table_extras_get_pixbuf(
+				ete, col_spec->pixbuf);
 			if (pixbuf) {
-				col = e_table_col_new_with_pixbuf (col_spec->model_col, pixbuf,
-								   col_spec->expansion, col_spec->minimum_width,
-								   cell, compare, col_spec->resizable);
+				col = e_table_col_new_with_pixbuf (
+					col_spec->model_col, gettext (col_spec->title),
+					pixbuf, col_spec->expansion,
+					col_spec->minimum_width,
+					cell, compare, col_spec->resizable);
 			}
-		} 
-		if (col == NULL && col_spec->title_ && *col_spec->title_) {
-			col = e_table_col_new (col_spec->model_col, col_spec->title_,
-					       col_spec->expansion, col_spec->minimum_width,
-					       cell, compare, col_spec->resizable);
+		}
+		if (col == NULL && col_spec->title && *col_spec->title) {
+			col = e_table_col_new (
+				col_spec->model_col, gettext (col_spec->title),
+				col_spec->expansion, col_spec->minimum_width,
+				cell, compare, col_spec->resizable);
 		}
 	}
 	return col;
 }
 
 static ETableHeader *
-et_spec_to_full_header (ETable *e_table, ETableSpecification *spec, ETableExtras *ete)
+et_spec_to_full_header (ETable              *e_table,
+			ETableSpecification *spec,
+			ETableExtras        *ete)
 {
 	ETableHeader *nh;
 	int column;
@@ -683,11 +768,12 @@ et_spec_to_full_header (ETable *e_table, ETableSpecification *spec, ETableExtras
 	g_return_val_if_fail (e_table, NULL);
 	g_return_val_if_fail (spec, NULL);
 	g_return_val_if_fail (ete, NULL);
-	
+
 	nh = e_table_header_new ();
 
 	for (column = 0; spec->columns[column]; column++) {
-		ETableCol *col = et_col_spec_to_col(e_table, spec->columns[column], ete);
+		ETableCol *col = et_col_spec_to_col (
+			e_table, spec->columns[column], ete);
 
 		if (col)
 			e_table_header_add_column (nh, col, -1);
@@ -706,7 +792,7 @@ et_state_to_header (ETable *e_table, ETableHeader *full_header, ETableState *sta
 	g_return_val_if_fail (e_table, NULL);
 	g_return_val_if_fail (full_header, NULL);
 	g_return_val_if_fail (state, NULL);
-	
+
 	nh = e_table_header_new ();
 
 	gtk_object_set(GTK_OBJECT(nh),
@@ -715,13 +801,21 @@ et_state_to_header (ETable *e_table, ETableHeader *full_header, ETableState *sta
 
 	for (column = 0; column < state->col_count; column++) {
 		int col;
+		double expansion;
+		ETableCol *table_col;
 
 		col = state->columns[column];
+		expansion = state->expansions[column];
 
 		if (col >= max_cols)
 			continue;
 
-		e_table_header_add_column (nh, e_table_header_get_column (full_header, col), -1);
+		table_col = e_table_header_get_column (full_header, col);
+
+		if (expansion >= -1)
+			table_col->expansion = expansion;
+
+		e_table_header_add_column (nh, table_col, -1);
 	}
 
 	return nh;
@@ -742,15 +836,16 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 					       e_table->group_info_change_id);
 		gtk_object_unref(GTK_OBJECT(e_table->sort_info));
 	}
-	e_table->sort_info = state->sort_info;
-	if (e_table->sort_info) {
-		gtk_object_ref(GTK_OBJECT(e_table->sort_info));
+	if (state->sort_info) {
+		e_table->sort_info = e_table_sort_info_duplicate(state->sort_info);
 		e_table->group_info_change_id =
 			gtk_signal_connect (GTK_OBJECT (e_table->sort_info),
 					    "group_info_changed",
 					    GTK_SIGNAL_FUNC (sort_info_changed),
 					    e_table);
 	}
+	else
+		e_table->sort_info = NULL;
 
 	if (e_table->sorter)
 		gtk_object_set(GTK_OBJECT(e_table->sorter),
@@ -771,8 +866,17 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 		e_table->rebuild_idle_id = g_idle_add_full (20, changed_idle, e_table, NULL);
 }
 
-void            e_table_set_state                 (ETable               *e_table,
-						   const gchar          *state_str)
+/**
+ * e_table_set_state:
+ * @e_table: %ETable object that will be modified
+ * @state_str: a string with the XML representation of the ETableState.
+ *
+ * This routine sets the state (as described by %ETableState) of the
+ * %ETable object.
+ */
+void
+e_table_set_state (ETable      *e_table,
+		   const gchar *state_str)
 {
 	ETableState *state;
 
@@ -789,8 +893,17 @@ void            e_table_set_state                 (ETable               *e_table
 	gtk_object_unref(GTK_OBJECT(state));
 }
 
-void            e_table_load_state                (ETable               *e_table,
-						   const gchar          *filename)
+/**
+ * e_table_load_state:
+ * @e_table: %ETable object that will be modified
+ * @filename: name of the file containing the state to be loaded into the %ETable
+ *
+ * An %ETableState will be loaded form the file pointed by @filename into the
+ * @e_table object.
+ */
+void
+e_table_load_state (ETable      *e_table,
+		    const gchar *filename)
 {
 	ETableState *state;
 
@@ -807,6 +920,13 @@ void            e_table_load_state                (ETable               *e_table
 	gtk_object_unref(GTK_OBJECT(state));
 }
 
+/**
+ * e_table_get_state_object:
+ * @e_table: %ETable object that will be modified
+ *
+ * Returns: the %ETableState object that encapsulates the current
+ * state of the @e_table object
+ */
 ETableState *
 e_table_get_state_object (ETable *e_table)
 {
@@ -818,19 +938,21 @@ e_table_get_state_object (ETable *e_table)
 	state->sort_info = e_table->sort_info;
 	gtk_object_ref(GTK_OBJECT(state->sort_info));
 
-	
+
 	state->col_count = e_table_header_count (e_table->header);
 	full_col_count = e_table_header_count (e_table->full_header);
 	state->columns = g_new(int, state->col_count);
+	state->expansions = g_new(double, state->col_count);
 	for (i = 0; i < state->col_count; i++) {
-		int col_idx = e_table_header_index(e_table->header, i);
+		ETableCol *col = e_table_header_get_column(e_table->header, i);
 		state->columns[i] = -1;
 		for (j = 0; j < full_col_count; j++) {
-			if (col_idx == e_table_header_index(e_table->full_header, j)) {
+			if (col->col_idx == e_table_header_index(e_table->full_header, j)) {
 				state->columns[i] = j;
 				break;
 			}
 		}
+		state->expansions[i] = col->expansion;
 	}
 
 	return state;
@@ -847,8 +969,17 @@ gchar          *e_table_get_state                 (ETable               *e_table
 	return string;
 }
 
-void            e_table_save_state                (ETable               *e_table,
-						   const gchar          *filename)
+/**
+ * e_table_save_state:
+ * @e_table: %ETable object that will be modified
+ * @filename: name of the file containing the state to be loaded into the %ETable
+ *
+ * This routine saves the state of the @e_table object into the file pointed
+ * by @filename
+ */
+void
+e_table_save_state (ETable      *e_table,
+		    const gchar *filename)
 {
 	ETableState *state;
 
@@ -876,11 +1007,17 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		ete = e_table_extras_new();
 
 	e_table->use_click_to_add = specification->click_to_add;
-	e_table->click_to_add_message = g_strdup(specification->click_to_add_message_);
+	e_table->click_to_add_message = g_strdup (gettext (specification->click_to_add_message));
 	e_table->draw_grid = specification->draw_grid;
+	e_table->draw_focus = specification->draw_focus;
 	e_table->cursor_mode = specification->cursor_mode;
 	e_table->full_header = et_spec_to_full_header(e_table, specification, ete);
 
+	gtk_object_set(GTK_OBJECT(e_table->selection),
+		       "selection_mode", specification->selection_mode,
+		       "cursor_mode", specification->cursor_mode,
+		       NULL);
+
 	e_table->model = etm;
 	gtk_object_ref (GTK_OBJECT (etm));
 
@@ -894,7 +1031,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "group_info_changed",
 				    GTK_SIGNAL_FUNC (sort_info_changed), e_table);
 
-	
+
 	gtk_object_set(GTK_OBJECT(e_table->header),
 		       "sort_info", e_table->sort_info,
 		       NULL);
@@ -933,7 +1070,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 			  GTK_FILL | GTK_EXPAND,
 			  GTK_FILL | GTK_EXPAND,
 			  0, 0);
-	
+
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
 
@@ -1037,7 +1174,7 @@ e_table_new (ETableModel *etm, ETableExtras *ete, const char *spec, const char *
 	e_table = gtk_type_new (e_table_get_type ());
 
 	e_table = e_table_construct (e_table, etm, ete, spec, state);
-		
+
 	return GTK_WIDGET (e_table);
 }
 
@@ -1054,7 +1191,7 @@ e_table_new_from_spec_file (ETableModel *etm, ETableExtras *ete, const char *spe
 	e_table = gtk_type_new (e_table_get_type ());
 
 	e_table = e_table_construct_from_spec_file (e_table, etm, ete, spec_fn, state_fn);
-		
+
 	return GTK_WIDGET (e_table);
 }
 
@@ -1086,7 +1223,7 @@ et_build_grouping_spec (ETable *e_table)
 	int i;
 	const int sort_count = e_table_sort_info_sorting_get_count (e_table->sort_info);
 	const int group_count = e_table_sort_info_grouping_get_count (e_table->sort_info);
-	
+
 	grouping = xmlNewNode (NULL, "grouping");
 	node = grouping;
 
@@ -1102,7 +1239,7 @@ et_build_grouping_spec (ETable *e_table)
 	for (i = 0; i < sort_count; i++) {
 		ETableSortColumn column = e_table_sort_info_sorting_get_nth(e_table->sort_info, i);
 		xmlNode *new_node = xmlNewChild(node, NULL, "leaf", NULL);
-		
+
 		e_xml_set_integer_prop_by_name (new_node, "column", column.column);
 		e_xml_set_integer_prop_by_name (new_node, "ascending", column.ascending);
 		node = new_node;
@@ -1120,7 +1257,7 @@ et_build_tree (ETable *e_table)
 	doc = xmlNewDoc ("1.0");
 	if (doc == NULL)
 		return NULL;
-	
+
 	root = xmlNewDocNode (doc, NULL, "ETableSpecification", NULL);
 	xmlDocSetRootElement (doc, root);
 	xmlAddChild (root, et_build_column_spec (e_table));
@@ -1248,7 +1385,7 @@ e_table_select_all (ETable *table)
 {
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (E_IS_TABLE (table));
-	
+
 	e_table_selection_model_select_all (table->selection);
 }
 
@@ -1257,7 +1394,7 @@ e_table_invert_selection (ETable *table)
 {
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (E_IS_TABLE (table));
-	
+
 	e_table_selection_model_invert_selection (table->selection);
 }
 
@@ -1274,11 +1411,8 @@ e_table_get_printable (ETable *e_table)
 static void
 et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
-	ETable *etable = E_TABLE (o);
-
 	switch (arg_id){
-	case ARG_TABLE_DRAW_FOCUS:
-		GTK_VALUE_BOOL (*arg) = etable->draw_focus;
+	default:
 		break;
 	}
 }
@@ -1292,7 +1426,7 @@ static void
 et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
 	ETable *etable = E_TABLE (o);
-	
+
 	switch (arg_id){
 	case ARG_LENGTH_THRESHOLD:
 		etable->length_threshold = GTK_VALUE_INT (*arg);
@@ -1302,18 +1436,10 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 					       NULL);
 		}
 		break;
-		
-	case ARG_TABLE_DRAW_FOCUS:
-		etable->draw_focus = GTK_VALUE_BOOL (*arg);
-		if (etable->group) {
-			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etable->group),
-					"drawfocus", GTK_VALUE_BOOL (*arg),
-					NULL);
-		}
-		break;
+
 	}
 }
-	
+
 static void
 set_scroll_adjustments   (ETable *table,
 			  GtkAdjustment *hadjustment,
@@ -1334,7 +1460,7 @@ set_scroll_adjustments   (ETable *table,
 					    hadjustment);
 }
 
-gint 
+gint
 e_table_get_next_row      (ETable *e_table,
 			   gint    model_row)
 {
@@ -1356,7 +1482,7 @@ e_table_get_next_row      (ETable *e_table,
 			return -1;
 }
 
-gint 
+gint
 e_table_get_prev_row      (ETable *e_table,
 			   gint    model_row)
 {
@@ -1401,7 +1527,7 @@ e_table_view_to_model_row        (ETable *e_table,
 		return view_row;
 }
 
-struct _ETableDragSourceSite 
+struct _ETableDragSourceSite
 {
 	GdkModifierType    start_button_mask;
 	GtkTargetList     *target_list;        /* Targets for drag data */
@@ -1409,24 +1535,24 @@ struct _ETableDragSourceSite
 	GdkColormap       *colormap;	         /* Colormap for drag icon */
 	GdkPixmap         *pixmap;             /* Icon for drag data */
 	GdkBitmap         *mask;
-	
+
 	/* Stored button press information to detect drag beginning */
 	gint               state;
 	gint               x, y;
 	gint               row, col;
 };
 
-typedef enum 
+typedef enum
 {
   GTK_DRAG_STATUS_DRAG,
   GTK_DRAG_STATUS_WAIT,
   GTK_DRAG_STATUS_DROP
 } GtkDragStatus;
 
-typedef struct _GtkDragDestInfo GtkDragDestInfo;  
+typedef struct _GtkDragDestInfo GtkDragDestInfo;
 typedef struct _GtkDragSourceInfo GtkDragSourceInfo;
 
-struct _GtkDragDestInfo 
+struct _GtkDragDestInfo
 {
   GtkWidget         *widget;	   /* Widget in which drag is in */
   GdkDragContext    *context;	   /* Drag context */
@@ -1441,7 +1567,7 @@ struct _GtkDragDestInfo
   gint               drop_x, drop_y; /* Position of drop */
 };
 
-struct _GtkDragSourceInfo 
+struct _GtkDragSourceInfo
 {
   GtkWidget         *widget;
   GtkTargetList     *target_list; /* Targets for drag data */
@@ -1460,7 +1586,7 @@ struct _GtkDragSourceInfo
   gint               cur_x, cur_y;     /* Current Position */
 
   GList             *selections;  /* selections we've claimed */
-  
+
   GtkDragDestInfo   *proxy_dest;  /* Set if this is a proxy drag */
 
   guint              drop_timeout;     /* Timeout for aborting drop */
@@ -1470,12 +1596,13 @@ struct _GtkDragSourceInfo
 
 /* Drag & drop stuff. */
 /* Target */
-void e_table_drag_get_data (ETable         *table,
-			    int             row,
-			    int             col,
-			    GdkDragContext *context,
-			    GdkAtom         target,
-			    guint32         time)
+void
+e_table_drag_get_data (ETable         *table,
+		       int             row,
+		       int             col,
+		       GdkDragContext *context,
+		       GdkAtom         target,
+		       guint32         time)
 {
 	g_return_if_fail(table != NULL);
 	g_return_if_fail(E_IS_TABLE(table));
@@ -1486,18 +1613,28 @@ void e_table_drag_get_data (ETable         *table,
 			  context,
 			  target,
 			  time);
-	
+
 }
 
-void e_table_drag_highlight   (ETable     *table,
-			       int         row,
-			       int         col) /* col == -1 to highlight entire row. */
+/**
+ * e_table_drag_highlight:
+ * @table:
+ * @row:
+ * @col:
+ *
+ * Set col to -1 to highlight the entire row.
+ */
+void
+e_table_drag_highlight (ETable *table,
+			int     row,
+			int     col)
 {
 	g_return_if_fail(table != NULL);
 	g_return_if_fail(E_IS_TABLE(table));
 }
 
-void e_table_drag_unhighlight (ETable     *table)
+void
+e_table_drag_unhighlight (ETable *table)
 {
 	g_return_if_fail(table != NULL);
 	g_return_if_fail(E_IS_TABLE(table));
@@ -1533,11 +1670,13 @@ void e_table_drag_dest_set_proxy (ETable         *table,
 				use_coordinates);
 }
 
-/* There probably should be functions for setting the targets
+/*
+ * There probably should be functions for setting the targets
  * as a GtkTargetList
  */
 
-void e_table_drag_dest_unset (GtkWidget          *widget)
+void
+e_table_drag_dest_unset (GtkWidget *widget)
 {
 	g_return_if_fail(widget != NULL);
 	g_return_if_fail(E_IS_TABLE(widget));
@@ -1547,11 +1686,12 @@ void e_table_drag_dest_unset (GtkWidget          *widget)
 
 /* Source side */
 
-void e_table_drag_source_set  (ETable               *table,
-			       GdkModifierType       start_button_mask,
-			       const GtkTargetEntry *targets,
-			       gint                  n_targets,
-			       GdkDragAction         actions)
+void
+e_table_drag_source_set  (ETable               *table,
+			  GdkModifierType       start_button_mask,
+			  const GtkTargetEntry *targets,
+			  gint                  n_targets,
+			  GdkDragAction         actions)
 {
 	ETableDragSourceSite *site;
 	GtkWidget *canvas;
@@ -1595,7 +1735,8 @@ void e_table_drag_source_set  (ETable               *table,
 	site->actions = actions;
 }
 
-void e_table_drag_source_unset (ETable        *table)
+void
+e_table_drag_source_unset (ETable *table)
 {
 	ETableDragSourceSite *site;
 
@@ -1605,9 +1746,13 @@ void e_table_drag_source_unset (ETable        *table)
 	site = table->site;
 
 	if (site) {
-		gtk_signal_disconnect (GTK_OBJECT (table->table_canvas), table->drag_source_button_press_event_id);
-		gtk_signal_disconnect (GTK_OBJECT (table->table_canvas), table->drag_source_motion_notify_event_id);
-		g_free(site);
+		gtk_signal_disconnect (
+			GTK_OBJECT (table->table_canvas),
+			table->drag_source_button_press_event_id);
+		gtk_signal_disconnect (
+			GTK_OBJECT (table->table_canvas),
+			table->drag_source_motion_notify_event_id);
+		g_free (site);
 		table->site = NULL;
 	}
 }
@@ -1639,15 +1784,13 @@ e_table_drag_begin (ETable            *table,
 }
 
 static void
-e_table_compute_location(ETable *table,
-			 GtkWidget *widget,
-			 int x,
-			 int y,
-			 int *row,
-			 int *col)
+e_table_compute_location (ETable *table, GtkWidget *widget,
+			  int x, int y, int *row, int *col)
 {
 	if (!(row || col))
 		return;
+	x += GTK_LAYOUT(table->table_canvas)->hadjustment->value;
+	y += GTK_LAYOUT(table->table_canvas)->vadjustment->value;
 	e_table_group_compute_location(table->group, &x, &y, row, col);
 }
 
@@ -1675,7 +1818,7 @@ et_drag_end (GtkWidget *widget,
 			 context);
 }
 
-static void 
+static void
 et_drag_data_get(GtkWidget *widget,
 		 GdkDragContext *context,
 		 GtkSelectionData *selection_data,
@@ -1693,7 +1836,7 @@ et_drag_data_get(GtkWidget *widget,
 			 time);
 }
 
-static void 
+static void
 et_drag_data_delete(GtkWidget *widget,
 		    GdkDragContext *context,
 		    ETable *et)
@@ -1705,7 +1848,7 @@ et_drag_data_delete(GtkWidget *widget,
 			 context);
 }
 
-static void 
+static void
 et_drag_leave(GtkWidget *widget,
 	      GdkDragContext *context,
 	      guint time,
@@ -1864,13 +2007,13 @@ e_table_drag_source_event_cb (GtkWidget      *widget,
 			}
 		}
 		break;
-      
+
 	case GDK_BUTTON_RELEASE:
 		if ((GDK_BUTTON1_MASK << (event->button.button - 1)) & site->start_button_mask) {
 			site->state &= ~(GDK_BUTTON1_MASK << (event->button.button - 1));
 		}
 		break;
-	  
+
 	case GDK_MOTION_NOTIFY:
 		if (site->state & event->motion.state & site->start_button_mask) {
 			/* FIXME: This is really broken and can leave us
@@ -1878,20 +2021,20 @@ e_table_drag_source_event_cb (GtkWidget      *widget,
 			 */
 			int i;
 			for (i=1; i<6; i++) {
-				if (site->state & event->motion.state & 
+				if (site->state & event->motion.state &
 				    GDK_BUTTON1_MASK << (i - 1))
 					break;
 			}
-		  
+
 			if (MAX (abs (site->x - event->motion.x),
 				 abs (site->y - event->motion.y)) > 3) {
 				GtkDragSourceInfo *info;
 				GdkDragContext *context;
-			  
+
 				site->state = 0;
 				context = e_table_drag_begin (table, site->row, site->col,
 							      site->target_list,
-							      site->actions, 
+							      site->actions,
 							      i, event);
 
 
@@ -1906,7 +2049,7 @@ e_table_drag_source_event_cb (GtkWidget      *widget,
 					else
 						gtk_drag_set_icon_default (context);
 				}
-			  
+
 				return TRUE;
 			}
 		}
@@ -1919,10 +2062,15 @@ e_table_drag_source_event_cb (GtkWidget      *widget,
 }
 
 static void
-e_table_class_init (GtkObjectClass *object_class)
+e_table_class_init (ETableClass *class)
 {
-	ETableClass *klass = E_TABLE_CLASS(object_class);
-	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(object_class);
+	GtkObjectClass *object_class;
+	GtkWidgetClass *widget_class;
+	GtkContainerClass *container_class;
+
+	object_class = (GtkObjectClass *) class;
+	widget_class = (GtkWidgetClass *) class;
+	container_class = (GtkContainerClass *) class;
 
 	e_table_parent_class = gtk_type_class (PARENT_TYPE);
 
@@ -1930,22 +2078,27 @@ e_table_class_init (GtkObjectClass *object_class)
 	object_class->set_arg           = et_set_arg;
 	object_class->get_arg           = et_get_arg;
 
-	klass->cursor_change            = NULL;
-	klass->selection_change         = NULL;
-	klass->double_click             = NULL;
-	klass->right_click              = NULL;
-	klass->click                    = NULL;
-	klass->key_press                = NULL;
+	widget_class->grab_focus = et_grab_focus;
 
-	klass->table_drag_begin         = NULL;
-	klass->table_drag_end           = NULL;
-	klass->table_drag_data_get      = NULL;
-	klass->table_drag_data_delete   = NULL;
+	container_class->focus = et_focus;
 
-	klass->table_drag_leave         = NULL;
-	klass->table_drag_motion        = NULL;
-	klass->table_drag_drop          = NULL;
-	klass->table_drag_data_received = NULL;
+	class->cursor_change            = NULL;
+	class->cursor_activated            = NULL;
+	class->selection_change         = NULL;
+	class->double_click             = NULL;
+	class->right_click              = NULL;
+	class->click                    = NULL;
+	class->key_press                = NULL;
+
+	class->table_drag_begin         = NULL;
+	class->table_drag_end           = NULL;
+	class->table_drag_data_get      = NULL;
+	class->table_drag_data_delete   = NULL;
+
+	class->table_drag_leave         = NULL;
+	class->table_drag_motion        = NULL;
+	class->table_drag_drop          = NULL;
+	class->table_drag_data_received = NULL;
 
 	et_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
@@ -1955,6 +2108,14 @@ e_table_class_init (GtkObjectClass *object_class)
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 
+	et_signals [CURSOR_ACTIVATED] =
+		gtk_signal_new ("cursor_activated",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (ETableClass, cursor_activated),
+				gtk_marshal_NONE__INT,
+				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+
 	et_signals [SELECTION_CHANGE] =
 		gtk_signal_new ("selection_change",
 				GTK_RUN_LAST,
@@ -2091,11 +2252,11 @@ e_table_class_init (GtkObjectClass *object_class)
 				GTK_TYPE_SELECTION_DATA,
 				GTK_TYPE_UINT,
 				GTK_TYPE_UINT);
-	
+
 	gtk_object_class_add_signals (object_class, et_signals, LAST_SIGNAL);
 
-	klass->set_scroll_adjustments = set_scroll_adjustments;
-	
+	class->set_scroll_adjustments = set_scroll_adjustments;
+
 	widget_class->set_scroll_adjustments_signal =
 		gtk_signal_new ("set_scroll_adjustments",
 				GTK_RUN_LAST,
@@ -2103,12 +2264,9 @@ e_table_class_init (GtkObjectClass *object_class)
 				GTK_SIGNAL_OFFSET (ETableClass, set_scroll_adjustments),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
-	
-	gtk_object_add_arg_type ("ETable::drawfocus", GTK_TYPE_BOOL,
-				 GTK_ARG_READWRITE, ARG_TABLE_DRAW_FOCUS);
+
 	gtk_object_add_arg_type ("ETable::length_threshold", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_LENGTH_THRESHOLD);
 }
 
 E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE);
-
