@@ -20,6 +20,9 @@ typedef struct {
 	
 	GnomeCanvas *canvas;
 	EText *item;
+	GtkJustification justification;
+
+	guint draw_borders : 1;
 } EEntry;
 
 typedef struct {
