@@ -3,7 +3,8 @@
 #define _GAL_DEFINE_VIEWS_MODEL_H_
 
 #include <gal/e-table/e-table-model.h>
-#include "gal-view.h"
+#include <gal/menus/gal-view.h>
+#include <gal/menus/gal-view-collection.h>
 
 #define GAL_DEFINE_VIEWS_MODEL_TYPE        (gal_define_views_model_get_type ())
 #define GAL_DEFINE_VIEWS_MODEL(o)          (GTK_CHECK_CAST ((o), GAL_DEFINE_VIEWS_MODEL_TYPE, GalDefineViewsModel))
@@ -11,19 +12,11 @@
 #define GAL_IS_DEFINE_VIEWS_MODEL(o)       (GTK_CHECK_TYPE ((o), GAL_DEFINE_VIEWS_MODEL_TYPE))
 #define GAL_IS_DEFINE_VIEWS_MODEL_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), GAL_DEFINE_VIEWS_MODEL_TYPE))
 
-/* Virtual Column list:
-   0   Email
-   1   Full Name
-   2   Street
-   3   Phone
-*/
-
 typedef struct {
 	ETableModel parent;
 
 	/* item specific fields */
-	GalView **data;
-	int data_count;
+	GalViewCollection *collection;
 
 	guint editable : 1;
 } GalDefineViewsModel;
@@ -34,10 +27,16 @@ typedef struct {
 } GalDefineViewsModelClass;
 
 
-GtkType      gal_define_views_model_get_type  (void);
-ETableModel *gal_define_views_model_new       (void);
+GtkType      gal_define_views_model_get_type     (void);
+ETableModel *gal_define_views_model_new          (void);
 
-void         gal_define_views_model_append    (GalDefineViewsModel *model,
-					       GalView             *view);
+void         gal_define_views_model_append       (GalDefineViewsModel *model,
+						  GalView             *view);
+GalView     *gal_define_views_model_get_view     (GalDefineViewsModel *model,
+						  int                  i);
+void         gal_define_views_model_delete_view  (GalDefineViewsModel *model,
+						  int                  i);
+void         gal_define_views_model_copy_view    (GalDefineViewsModel *model,
+						  int                  i);
 
 #endif /* _GAL_DEFINE_VIEWS_MODEL_H_ */
