@@ -9,6 +9,7 @@
  *
  */
 #include <config.h>
+#include "gal/util/e-i18n.h"
 #include <gnome.h>
 #include <libgnomeui/gnome-canvas.h>
 #include "color-group.h"
@@ -379,13 +380,14 @@ color_palette_setup (ColorPalette *P,
 							P->custom_color_pos = total;
 						}
 						P->items[total] =
-							color_palette_button_new(P,
-										 GTK_TABLE (table),
-										 GTK_TOOLTIPS (tool_tip),
-										 &(color_name),
-										 col,
-										 row + 1,
-										 total);
+							color_palette_button_new(
+								P,
+								GTK_TABLE (table),
+								GTK_TOOLTIPS (tool_tip),
+								&(color_name),
+								col,
+								row + 1,
+								total);
 						total++;
 					}
 				}
@@ -394,14 +396,15 @@ color_palette_setup (ColorPalette *P,
 				break;
 			}
 			
-			P->items[total] =
-				color_palette_button_new(P,
-							 GTK_TABLE (table),
-							 GTK_TOOLTIPS (tool_tip),
-							 &(color_names [pos]),
-							 col,
-							 row + 1,
-							 total);
+			P->items [total] =
+				color_palette_button_new (
+					P,
+					GTK_TABLE (table),
+					GTK_TOOLTIPS (tool_tip),
+					&(color_names [pos]),
+					col,
+					row + 1,
+					total);
 			total++;
 		}
 	}
