@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 #define _E_FONT_C_
 
 /*
@@ -9,10 +10,9 @@
  *
  * Copyright (C) 2000 Helix Code, Inc.
  *
- * TODO: LRU Cache
- *
  */
 
+#include "config.h"
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -21,9 +21,31 @@
 #include "gal/util/e-cache.h"
 #include "e-font.h"
 
-#define noE_FONT_VERBOSE
+#ifdef HAVE_CODESET
+#include <langinfo.h>
+#endif
+
+static int e_font_verbose = 0;
+
+#define E_FONT_VERBOSE
 #define E_FONT_CACHE_SIZE 32
 
+enum {
+	E_XF_FOUNDRY,
+	E_XF_FAMILY,
+	E_XF_WEIGHT,
+	E_XF_SLANT,
+	E_XF_SET_WIDTH,
+	E_XF_ADD_STYLE,
+	E_XF_PIXEL_SIZE,
+	E_XF_POINT_SIZE,
+	E_XF_RESOLUTION_X,
+	E_XF_RESOLUTION_Y,
+	E_XF_SPACING,
+	E_XF_AVERAGE_WIDTH,
+	E_XF_CHARSET
+};
+
 struct _EFont {
 	gint refcount;
 	GdkFont *font;
@@ -34,6 +56,7 @@ struct _EFont {
 	iconv_t from;
 };
 
+static EFont * e_font_from_gdk_fontset (GdkFont *gdkfont);
 static gchar * get_font_name (const GdkFont * font);
 static void split_name (gchar * c[], gchar * name);
 static gboolean find_variants (gchar **namelist, gint length,
@@ -48,13 +71,82 @@ static const gchar *locale_charset_to_encoding(void);
 static void e_font_print_gdk_font_name (const GdkFont * font);
 #endif
 
+/*
+ * Creates EFont, given GdkFont, just like gdk_font(set)_load would do
+ */
+
 EFont *
 e_font_from_gdk_name (const gchar *name)
 {
 	EFont * font;
 	GdkFont *gdkfont;
+	gboolean need_fontset = FALSE;
+	gchar n[1024];
 
-	gdkfont = gdk_font_load (name);
+	g_return_val_if_fail (name != NULL, NULL);
+
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("(e_font_from_gdk_name) Requested: %s\n", name);
+	}
+#endif
+
+	/* Check whether we need font or fontset here */
+	if (strchr (name, ',')) {
+		/* Name is already fontset name */
+		need_fontset = TRUE;
+		g_snprintf (n, 1024, name);
+	} else {
+		gchar *c[14], *nn;
+		gint i;
+		/* Split name into pieces */
+		for (i = 0; i < 14; i++) c[i] = NULL;
+		nn = g_strdup (name);
+		split_name (c, nn);
+		if (c[E_XF_CHARSET] != NULL) {
+			gint len;
+			/* Font name has charset */
+			/* Determine length of base encoding part */
+			if (strchr (c[E_XF_CHARSET], '.')) {
+				len = strchr (c[E_XF_CHARSET], '.') - c[E_XF_CHARSET];
+			} else {
+				len = strlen (c[E_XF_CHARSET]);
+			}
+			/* Test well-known cases */
+			/* fixme: Whoever actually uses CJK locale should check/change that */
+			if (!strncasecmp ("eucjp", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("ujis", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("jisx208", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("gb2312", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("ksc5601", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+		}
+		g_free (nn);
+		if (need_fontset) {
+			/* We do not have ',', so we have to extend name */
+			g_snprintf (n, 1024, "%s,*", name);
+		} else {
+			/* Simply duplicate name */
+			g_snprintf (n, 1024, name);
+		}
+	}
+
+	/*
+	 * Here we are:
+	 * n is actual name to be loaded
+	 * need_fontset determines, whether we need fontset or not
+	 */
+
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("(e_font_from_gdk_name) Actual: %s\n", n);
+	}
+#endif
+
+	if (need_fontset) {
+		gdkfont = gdk_fontset_load (n);
+	} else {
+		gdkfont = gdk_font_load (n);
+	}
 	if (!gdkfont) gdkfont = gdk_font_load ("fixed");
 	g_return_val_if_fail (gdkfont != NULL, NULL);
 	font = e_font_from_gdk_font (gdkfont);
@@ -74,6 +166,14 @@ set_nbsp_zero_width_flag (EFont *efont)
 		? FALSE : TRUE;
 }
 
+/*
+ * Creates EFont from existing GdkFont.
+ * 1. Use cache
+ * 2. Analyze, whether we need font or fontset. For fontset use e_font_from_gdk_fontset
+ * 3. Try, if we can simply use iso-10646 variant of font
+ * 4. Determine bold or light variant
+ */
+
 EFont *
 e_font_from_gdk_font (GdkFont *gdkfont)
 {
@@ -83,6 +183,15 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 	gchar * name;
 	XFontStruct *xfs;
 
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("(e_font_from_gdk_font): Initial font:\n");
+		e_font_print_gdk_font_name (gdkfont);
+	}
+#endif
+
+	g_return_val_if_fail (gdkfont != NULL, NULL);
+
 	/* Try cache */
 	if (!cache) cache = e_cache_new (NULL, /* Key hash */
 					 NULL, /* Key compare */
@@ -92,144 +201,175 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 					 E_FONT_CACHE_SIZE, /* Soft limit (fake) */
 					 E_FONT_CACHE_SIZE); /* Hard limit */
 
-	font = e_cache_lookup (cache, gdkfont);
-	if (font) {
+	if ((font = e_cache_lookup (cache, gdkfont))) {
+		/* Use cached font, increase refcount and return */
 		e_font_ref (font);
 		return font;
 	} else {
+		/* Save original font to be used as cache key */
 		keyfont = gdkfont;
 	}
 
-	/* Not in cache */
+	if (gdkfont->type == GDK_FONT_FONTSET) {
+		gchar *xlocale[] = {	"ja_JP.euc", "ja_JP.ujis",
+					"zh_CN.GB2312", "zh_TW.Big5",
+					"ko_KR.eucKR", NULL};
+		char * fslocale;
+		int i;
+
+		fslocale = XLocaleOfFontSet (GDK_FONT_XFONT (gdkfont));
+		/* fixme: Whoever uses CJK locales should check that */
+		for (i = 0; xlocale[i] != NULL; i++) {
+			if ( g_strncasecmp (fslocale, xlocale[i], strlen (xlocale[i])) == 0) {
+				/* We need fontset */
+				font = e_font_from_gdk_fontset (gdkfont);
+				/* Insert into cache */
+				if (e_cache_insert (cache, keyfont, font, 1)) {
+					/* Inserted, so add cache ref manually */
+					e_font_ref (font);
+				}
+				return font;
+			}
+		}
+	}
+
+	/* We can do with simple font */
+	/* Fill out emergency values */
 	lightfont = gdkfont;
 	boldfont = NULL;
 
+	/* ref in case we cannot find any better variants */
 	gdk_font_ref (gdkfont);
 
+	/* Get full-qualified font name */
 	name = get_font_name (gdkfont);
 
 	if (name) {
-		gchar *c[14], *p;
+		gchar p[1024];
+		gchar *c[14];
 		const gchar * encoding;
 		gchar *boldname, *lightname;
 		gchar **namelist;
 		GdkFont *newfont;
-		gint numfonts, len;
-
-		/* Allocate space for experimenting */
-
-		len = strlen (name) + 64; /* Hope that's sufficent */
-		p = alloca (len);
+		gint numfonts;
 
 		/* Split name into components */
-
 		split_name (c, name);
 
 		/* Try to find iso-10646-1 encoded font with same name */
 		/* Compose name for unicode encoding */
+		/* fixme: We require same pixel size, I do not know, whether that is correct */
 		encoding = "iso10646-1";
-		g_snprintf (p, len, "%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s",
-			    "*", c[1], c[2], c[3], c[4], c[5], c[6], c[7],
-			    c[8], c[9], c[10], c[11], c[12], encoding);
+		g_snprintf (p, 1024, "-*-%s-%s-%s-%s-*-%s-*-*-*-*-*-%s",
+			    c[E_XF_FAMILY],
+			    c[E_XF_WEIGHT],
+			    c[E_XF_SLANT],
+			    c[E_XF_SET_WIDTH],
+			    c[E_XF_PIXEL_SIZE],
+			    encoding);
 		/* Try to load unicode font */
 #ifdef E_FONT_VERBOSE
-		g_print ("Trying font: %s\n", p);
+		if (e_font_verbose) {
+			g_print ("Trying unicode font: %s\n", p);
+		}
 #endif
 		newfont = gdk_font_load (p);
 		if (newfont) {
 #ifdef E_FONT_VERBOSE
-			e_font_print_gdk_font_name (newfont);
+			if (e_font_verbose) {
+				e_font_print_gdk_font_name (newfont);
+			}
 #endif
-			/* OK, use that */
+			/* OK, use found iso-10646 font */
+			/* Unref original font */
 			gdk_font_unref (gdkfont);
 			gdkfont = newfont;
 		} else {
-			/* Try to load fontset */
-			g_snprintf (p, len, "%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s,*",
-				    "*", c[1], c[2], c[3], c[4], c[5], c[6], c[7],
-				    c[8], c[9], c[10], c[11], c[12], c[13]);
-#ifdef E_FONT_VERBOSE
-			g_print ("Trying fontset: %s\n", p);
-#endif
-			newfont = gdk_fontset_load (p);
-
-			if (newfont) {
-#ifdef E_FONT_VERBOSE
-				e_font_print_gdk_font_name (newfont);
-#endif
-				/* OK, use fontset */
-				gdk_font_unref (gdkfont);
-				gdkfont = newfont;
-				encoding = "*";
-			} else {
-				/* Nope, use original font */
-				encoding = c[13];
-			}
+			encoding = c[E_XF_CHARSET];
 		}
 
-		/* Here we are:
-		 * gdkfont points to font(set) to be used, with right refcount
-		 * encoding is the X encoding string for that font
+		/*
+		 * Here we are:
+		 * gdkfont - font to be used, with right refcount
+		 * encoding - X encoding string for that font
 		 */
 
-		/* List all weight variants of given font(set) */
-
-		g_snprintf (p, len, "%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s",
-			    "*", c[1], c[2], "*", c[4], c[5], "*", "*",
-			    c[8], c[9], c[10], c[11], "*", encoding);
+		/* Read all weight variants of given font */
+		g_snprintf (p, 1024, "-*-%s-*-%s-%s-*-%s-*-*-*-*-*-%s",
+			    c[E_XF_FAMILY],
+			    c[E_XF_SLANT],
+			    c[E_XF_SET_WIDTH],
+			    c[E_XF_PIXEL_SIZE],
+			    encoding);
 		namelist = XListFonts (GDK_FONT_XDISPLAY (gdkfont), p, 32, &numfonts);
 
-		if (find_variants (namelist, numfonts, c[3], &lightname, &boldname)) {
-			gboolean is_fontset;
-
-			is_fontset = (gdkfont->type == GDK_FONT_FONTSET);
-
-			/* There are usable variants - their weights are returned in lightname and boldname */
+		lightname = boldname = NULL;
+		if (namelist &&
+		    numfonts &&
+		    find_variants (namelist, numfonts, c[E_XF_WEIGHT], &lightname, &boldname) &&
+		    lightname &&
+		    boldname) {
+			/* There are usable weight variants - their weights are returned in lightname and boldname */
 			lightfont = NULL;
 			boldfont  = NULL;
-			if (!g_strcasecmp (c[3], lightname))
+			if (!g_strcasecmp (c[E_XF_WEIGHT], lightname)) {
 				/* Our name maps to light font */
 				lightfont = gdkfont;
-			else if (!g_strcasecmp (c[3], boldname))
+			} else if (!g_strcasecmp (c[E_XF_WEIGHT], boldname)) {
 				/* Our name maps to bold font */
 				boldfont = gdkfont;
-			else
+			} else {
 				/* Our name maps neither to light nor to bold font */
 				gdk_font_unref (gdkfont);
+				gdkfont = NULL;
+			}
 
 			if (!lightfont) {
-				g_snprintf (p, len, "%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s%s",
-					    "*", c[1], c[2], lightname, c[4],
-					    c[5], "*", "*", c[8], c[9], c[10],
-					    c[11], "*", encoding, is_fontset ? ",*" : "");
+				/* We have to load lighter variant */
+				g_snprintf (p, 1024, "-*-%s-%s-%s-%s-*-%s-*-*-*-*-*-%s",
+					    c[E_XF_FAMILY],
+					    lightname,
+					    c[E_XF_SLANT],
+					    c[E_XF_SET_WIDTH],
+					    c[E_XF_PIXEL_SIZE],
+					    encoding);
 #ifdef E_FONT_VERBOSE
-				g_print ("Trying light: %s\n", p);
-#endif
-				if (is_fontset) {
-					lightfont = gdk_fontset_load (p);
-				} else {
-					lightfont = gdk_font_load (p);
+				if (e_font_verbose) {
+					g_print ("Trying light: %s\n", p);
 				}
+#endif
+				lightfont = gdk_font_load (p);
 #ifdef E_FONT_VERBOSE
-				e_font_print_gdk_font_name (lightfont);
+				if (e_font_verbose) {
+					e_font_print_gdk_font_name (lightfont);
+				}
 #endif
 			}
 			if (!boldfont) {
-				g_snprintf (p, len, "%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s%s",
-					    "*", c[1], c[2], boldname, c[4],
-					    c[5], "*", "*", c[8], c[9], c[10],
-					    c[11], "*", encoding, is_fontset ? ",*" : "");
+				/* We have to load bolder variant */
+				g_snprintf (p, 1024, "-*-%s-%s-%s-%s-*-%s-*-*-*-*-*-%s",
+					    c[E_XF_FAMILY],
+					    boldname,
+					    c[E_XF_SLANT],
+					    c[E_XF_SET_WIDTH],
+					    c[E_XF_PIXEL_SIZE],
+					    encoding);
 #ifdef E_FONT_VERBOSE
-				g_print ("Trying bold: %s\n", p);
-#endif
-				if (is_fontset) {
-					boldfont = gdk_fontset_load (p);
-				} else {
-					boldfont = gdk_font_load (p);
+				if (e_font_verbose) {
+					g_print ("Trying bold: %s\n", p);
 				}
-#ifdef E_FONT_VERBOSE
-				e_font_print_gdk_font_name (boldfont);
 #endif
+				boldfont = gdk_font_load (p);
+#ifdef E_FONT_VERBOSE
+				if (e_font_verbose) {
+					e_font_print_gdk_font_name (boldfont);
+				}
+#endif
+			}
+			/* I am not sure, whether that can happen, but let's be cautious */
+			if (!lightfont) {
+				lightfont = keyfont;
+				gdk_font_ref (lightfont);
 			}
 		} else {
 			/* We fall back to double drawing */
@@ -243,9 +383,11 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 	}
 
 #ifdef E_FONT_VERBOSE
-	g_print ("******** Creating EFont with following fonts ********\n");
-	e_font_print_gdk_font_name (lightfont);
-	e_font_print_gdk_font_name (boldfont);
+	if (e_font_verbose) {
+		g_print ("******** Creating EFont with following fonts ********\n");
+		e_font_print_gdk_font_name (lightfont);
+		e_font_print_gdk_font_name (boldfont);
+	}
 #endif
 
 	/* Here we are:
@@ -260,7 +402,7 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 	font->refcount = 1;
 	font->font = lightfont;
 	font->bold = boldfont;
-	font->twobyte = (lightfont->type == GDK_FONT_FONTSET || ((lightfont->type == GDK_FONT_FONT) && ((xfs->min_byte1 != 0) || (xfs->max_byte1 != 0))));
+	font->twobyte = (lightfont->type == GDK_FONT_FONTSET || ((xfs->min_byte1 != 0) || (xfs->max_byte1 != 0)));
 	font->to = e_iconv_to_gdk_font (font->font);
 	font->from = e_iconv_from_gdk_font (font->font);
 	set_nbsp_zero_width_flag (font);
@@ -275,6 +417,179 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 
 }
 
+GdkFont *
+e_font_to_gdk_font (EFont *font, EFontStyle style)
+{
+	if (style & E_FONT_BOLD) {
+		gdk_font_ref(font->bold);
+		return font->bold;
+	} else {
+		gdk_font_ref(font->font);
+		return font->font;
+	}
+}
+
+/*
+ * Creates EFont from GdkFontset, if locale needs fontset
+ */
+
+static EFont *
+e_font_from_gdk_fontset (GdkFont *gdkfont)
+{
+	EFont *font;
+	GdkFont *boldfont, *lightfont;
+	gchar * name;
+	XFontStruct *xfs;
+
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("(e_font_from_gdk_fontset): Initial font:\n");
+		e_font_print_gdk_font_name (gdkfont);
+	}
+#endif
+
+	lightfont = gdkfont;
+	boldfont = NULL;
+
+	/* fixme: why we ref here? */
+	gdk_font_ref (gdkfont);
+
+	/* Get full-qualified font name */
+	name = get_font_name (gdkfont);
+
+	if (name) {
+		gchar *c[14];
+		gchar p[1024];
+		const gchar * encoding;
+		gchar *boldname, *lightname;
+		gchar **namelist;
+		gint numfonts;
+
+		/* Split name into components */
+		split_name (c, name);
+
+		encoding = c[E_XF_CHARSET];
+
+		/* Here we are:
+		 * gdkfont points to fontset to be used, with right refcount
+		 * encoding is the X encoding string for that font
+		 */
+
+		/* Read all weight variants of base font */
+		g_snprintf (p, 1024, "-*-%s-*-%s-%s-*-%s-*-*-*-*-*-%s",
+			    c[E_XF_FAMILY],
+			    c[E_XF_SLANT],
+			    c[E_XF_SET_WIDTH],
+			    c[E_XF_PIXEL_SIZE],
+			    encoding);
+		namelist = XListFonts (GDK_FONT_XDISPLAY (gdkfont), p, 32, &numfonts);
+
+		if (namelist &&
+		    numfonts &&
+		    find_variants (namelist, numfonts, c[E_XF_WEIGHT], &lightname, &boldname) &&
+		    lightname &&
+		    boldname) {
+			/* There are usable variants - their weights are returned in lightname and boldname */
+			lightfont = NULL;
+			boldfont  = NULL;
+			if (!g_strcasecmp (c[E_XF_WEIGHT], lightname)) {
+				/* Our name maps to light font */
+				lightfont = gdkfont;
+				gdk_font_ref (gdkfont);
+			} else if (!g_strcasecmp (c[E_XF_WEIGHT], boldname)) {
+				/* Our name maps to bold font */
+				boldfont = gdkfont;
+				gdk_font_ref (gdkfont);
+			}
+
+			if (!lightfont) {
+				/* We have to load lighter variant */
+				g_snprintf (p, 1024, "-*-%s-%s-%s-%s-*-%s-*-*-*-*-*-%s,*",
+					    c[E_XF_FAMILY],
+					    lightname,
+					    c[E_XF_SLANT],
+					    c[E_XF_SET_WIDTH],
+					    c[E_XF_PIXEL_SIZE],
+					    encoding);
+#ifdef E_FONT_VERBOSE
+				if (e_font_verbose) {
+					g_print ("Trying light: %s\n", p);
+				}
+#endif
+				lightfont = gdk_fontset_load (p);
+#ifdef E_FONT_VERBOSE
+				if (e_font_verbose) {
+					e_font_print_gdk_font_name (lightfont);
+				}
+#endif
+			}
+			if (!boldfont) {
+				/* We have to load lighter variant */
+				g_snprintf (p, 1024, "-*-%s-%s-%s-%s-*-%s-*-*-*-*-*-%s,*",
+					    c[E_XF_FAMILY],
+					    boldname,
+					    c[E_XF_SLANT],
+					    c[E_XF_SET_WIDTH],
+					    c[E_XF_PIXEL_SIZE],
+					    encoding);
+#ifdef E_FONT_VERBOSE
+				if (e_font_verbose) {
+					g_print ("Trying bold: %s\n", p);
+				}
+#endif
+				boldfont = gdk_fontset_load (p);
+#ifdef E_FONT_VERBOSE
+				if (e_font_verbose) {
+					e_font_print_gdk_font_name (boldfont);
+				}
+#endif
+			}
+			if (!lightfont) {
+				lightfont = gdkfont;
+				gdk_font_ref (lightfont);
+			} else {
+				gdk_font_unref (gdkfont);
+			}
+		} else {
+			/* We fall back to double drawing */
+			lightfont = gdkfont;
+			boldfont = NULL;
+		}
+
+		XFreeFontNames (namelist);
+
+		g_free (name);
+	}
+
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("******** Creating EFont with following fonts ********\n");
+		e_font_print_gdk_font_name (lightfont);
+		e_font_print_gdk_font_name (boldfont);
+	}
+#endif
+
+	/* Here we are:
+	 * lightfont is light GdkFont with correct refcount
+	 * boldfont is either bold GdkFont or NULL
+	 */
+
+	font = g_new (EFont, 1);
+
+	xfs = GDK_FONT_XFONT (lightfont);
+
+	font->refcount = 1;
+	font->font = lightfont;
+	font->bold = boldfont;
+	font->twobyte = (lightfont->type == GDK_FONT_FONTSET || ((xfs->min_byte1 != 0) || (xfs->max_byte1 != 0)));
+	font->to = e_iconv_to_gdk_font (font->font);
+	font->from = e_iconv_from_gdk_font (font->font);
+	set_nbsp_zero_width_flag (font);
+
+	return font;
+
+}
+
 gchar *
 e_font_get_name (EFont *font)
 {
@@ -380,7 +695,7 @@ replace_nbsp_with_spaces (const guchar *orig, gint *bytes)
 static gint
 e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
 {
-	const char *ib;
+	char *ib;
 	char *ob;
 	size_t ibl, obl;
 	gint rv;
@@ -391,7 +706,7 @@ e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
 	if (font->to == (iconv_t) -1)
 		rv = no_conv_wrapper (font, native, utf, bytes);
 	else {
-		ib = utf;
+		ib = (char *) utf;
 		ibl = bytes;
 		ob = native;
 		obl = bytes * 4;
@@ -506,15 +821,9 @@ translate_encoding (const gchar *encoding)
 {
 	static GHashTable *eh = NULL;
 	gchar e[64];
+	gchar *finalenc;
 
-#if 0
-	static const gchar *charset;
-
-	charset = get_locale_charset();
-	if (charset) return charset;
-#endif
-
-	if (!eh) {
+	if (eh == NULL) {
 		eh = g_hash_table_new (g_str_hash, g_str_equal);
 
 		g_hash_table_insert (eh, "iso8859-1", "iso-8859-1");
@@ -543,15 +852,20 @@ translate_encoding (const gchar *encoding)
 		g_hash_table_insert (eh, "iso_8859-14", "iso-8859-14");
 		g_hash_table_insert (eh, "iso8859-15", "iso-8859-15");
 		g_hash_table_insert (eh, "iso_8859-15", "iso-8859-15");
-		g_hash_table_insert (eh, "iso10646-1", "UTF-16");
-		g_hash_table_insert (eh, "iso_10646-1", "UTF-16");
+		g_hash_table_insert (eh, "iso10646-1", "iso-10646-1");
+		g_hash_table_insert (eh, "iso_10646-1", "iso-10646-1");
 		g_hash_table_insert (eh, "koi8-r", "koi8-r");
+		g_hash_table_insert (eh, "koi8-ub", "koi8-ub");
+		g_hash_table_insert (eh, "eucjp", "EUC-JP");
+		g_hash_table_insert (eh, "ujis", "EUC-JP");
 	}
 
 	strncpy (e, encoding, 64);
 	g_strdown (e);
 
-	return g_hash_table_lookup (eh, e);
+	finalenc = g_hash_table_lookup (eh, e);
+
+	return finalenc ? finalenc : encoding;
 }
 
 static const gchar *
@@ -580,6 +894,12 @@ get_locale_charset (void)
 		initialized = TRUE;
 	}
 
+#ifdef E_FONT_VERBOSE
+	if (e_font_verbose) {
+		g_print ("Locale charset is: %s\n", charset);
+	}
+#endif
+
 	return charset;
 }
 
@@ -602,6 +922,19 @@ locale_charset_to_encoding(void)
 }
 #endif
 
+const gchar *
+e_locale_encoding (void)
+{
+#ifdef HAVE_CODESET
+	const char *encoding = nl_langinfo(CODESET);
+	if (encoding) {
+		const gchar *trans_encoding = translate_encoding (encoding);
+		if (trans_encoding) return trans_encoding;
+	}
+#endif
+	return NULL;
+}
+
 const gchar *
 e_gdk_font_encoding (GdkFont *font)
 {
@@ -626,16 +959,17 @@ e_gdk_font_encoding (GdkFont *font)
 	if (encoding) return encoding;
 
 #ifdef E_FONT_VERBOSE
-	g_print ("Extracting X font info\n");
+	if (e_font_verbose) {
+		g_print ("Extracting X font info\n");
+	}
 #endif
 
 	if (font->type == GDK_FONT_FONTSET) {
 		/* We are fontset, so try to find right encoding from locale */
 		encoding = get_locale_charset ();
 		if (encoding) {
-		  const gchar *trans_encoding = translate_encoding (encoding);
-
-		  if (trans_encoding) return trans_encoding;
+			const gchar *trans_encoding = translate_encoding (encoding);
+			if (trans_encoding) return trans_encoding;
 		}
 	}
 
@@ -686,17 +1020,12 @@ e_gdk_font_encoding (GdkFont *font)
 	return encoding;
 }
 
-iconv_t
-e_iconv_from_gdk_font (GdkFont *font)
+static iconv_t
+e_iconv_from_encoding (const gchar *enc)
 {
 	static GHashTable *ih = NULL;
-	const gchar *enc;
 	iconv_t ic;
 
-	if (!font) return (iconv_t) -1;
-
-	enc = e_gdk_font_encoding (font);
-
 	if (!enc) return (iconv_t) -1;
 
 	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
@@ -712,17 +1041,12 @@ e_iconv_from_gdk_font (GdkFont *font)
 	return ic;
 }
 
-iconv_t
-e_iconv_to_gdk_font (GdkFont *font)
+static iconv_t
+e_iconv_to_encoding (const gchar *enc)
 {
 	static GHashTable *ih = NULL;
-	const gchar *enc;
 	iconv_t ic;
 
-	if (!font) return (iconv_t) -1;
-
-	enc = e_gdk_font_encoding (font);
-
 	if (!enc) return (iconv_t) -1;
 
 	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
@@ -738,6 +1062,48 @@ e_iconv_to_gdk_font (GdkFont *font)
 	return ic;
 }
 
+iconv_t
+e_iconv_from_locale (void)
+{
+	const gchar *enc;
+
+	enc = e_locale_encoding ();
+	return e_iconv_from_encoding (enc);
+}
+
+iconv_t
+e_iconv_to_locale (void)
+{
+	const gchar *enc;
+
+	enc = e_locale_encoding ();
+	return e_iconv_to_encoding (enc);
+}
+
+iconv_t
+e_iconv_from_gdk_font (GdkFont *font)
+{
+	const gchar *enc;
+
+	if (!font) return (iconv_t) -1;
+
+	enc = e_gdk_font_encoding (font);
+
+	return e_iconv_from_encoding (enc);
+}
+
+iconv_t
+e_iconv_to_gdk_font (GdkFont *font)
+{
+	const gchar *enc;
+
+	if (!font) return (iconv_t) -1;
+
+	enc = e_gdk_font_encoding (font);
+
+	return e_iconv_to_encoding (enc);
+}
+
 /*
  * Return newly allocated full name
  */
@@ -749,7 +1115,9 @@ get_font_name (const GdkFont * font)
 	Bool status;
 
 #ifdef E_FONT_VERBOSE
-	g_print ("Extracting X font info\n");
+	if (e_font_verbose) {
+		g_print ("Extracting X font info\n");
+	}
 #endif
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
@@ -760,6 +1128,15 @@ get_font_name (const GdkFont * font)
 		gchar **font_names;
 
 		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
+#ifdef E_FONT_VERBOSE
+		if (e_font_verbose) {
+			gint i;
+
+			g_print ("Fonts of fontset:\n");
+			for (i = 0; i < num_fonts; i++)
+				g_print ("  %s\n", font_names[i]);
+		}
+#endif
 		status = XGetFontProperty (font_structs[0], font_atom, &atom);
 	} else {
 		status = XGetFontProperty (GDK_FONT_XFONT (font), font_atom, &atom);
@@ -783,8 +1160,9 @@ split_name (gchar * c[], gchar * name)
 	gint i;
 
 	p = name;
+	if (*p == '-') p++;
 
-	for (i = 0; i < 13; i++) {
+	for (i = 0; i < 12; i++) {
 		c[i] = p;
 		/* Skip text */
 		while (*p && (*p != '-')) p++;
@@ -795,7 +1173,8 @@ split_name (gchar * c[], gchar * name)
 	c[i] = p;
 }
 
-/* Find light and bold variants of a font, ideally using the provided
+/*
+ * Find light and bold variants of a font, ideally using the provided
  * weight for the light variant, and a weight 2 shades darker than it
  * for the bold variant. If there isn't something 2 shades darker, use
  * something 3 or more shades darker if it exists, or 1 shade darker
@@ -803,6 +1182,7 @@ split_name (gchar * c[], gchar * name)
  * weight, but there are lighter fonts, then use the darker one for
  * bold and a lighter one for light.
  */
+
 static gboolean
 find_variants (gchar **namelist, gint length, gchar *weight,
 	       gchar **lightname, gchar **boldname)
@@ -810,7 +1190,8 @@ find_variants (gchar **namelist, gint length, gchar *weight,
 	static GHashTable *wh = NULL;
 	/* Standard, Found, Bold, Light */
 	gint sw, fw, bw, lw;
-	gchar *s, *f, *b, *l;
+	gchar s[32];
+	gchar *f, *b, *l;
 	gchar *p;
 	gint i;
 
@@ -825,8 +1206,7 @@ find_variants (gchar **namelist, gint length, gchar *weight,
 		g_hash_table_insert (wh, "black", GINT_TO_POINTER (8));
 	}
 
-	s = alloca (strlen (weight) + 1);
-	strcpy (s, weight);
+	g_snprintf (s, 32, weight);
 	g_strdown (s);
 	sw = GPOINTER_TO_INT (g_hash_table_lookup (wh, s));
 	if (sw == 0) return FALSE;
@@ -903,7 +1283,7 @@ e_font_print_gdk_font_name (const GdkFont * font)
 
 		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
 
-		g_print ("Gdk Fontset:\n");
+		g_print ("Gdk Fontset, locale: %s\n", XLocaleOfFontSet (GDK_FONT_XFONT (font)));
 		for (i = 0; i < num_fonts; i++) {
 			g_print ("    %s\n", font_names[i]);
 		}
