@@ -1,6 +1,16 @@
 #ifndef _E_UNICODE_H_
 #define _E_UNICODE_H_
 
+/*
+ * UTF-8 support functions for gal
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@helixcode.com>
+ *
+ * Copyright (C) 2000-2001 Helix Code, Inc.
+ *
+ */
+
 #include <sys/types.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -15,6 +25,8 @@ BEGIN_GNOME_DECLS
 #define gnome_font_lookup_default gnome_font_get_glyph
 #endif
 
+#define G_UTF8_IN_GAL
+
 void e_unicode_init (void);
 
 /*
@@ -34,9 +46,14 @@ gchar *e_utf8_from_gtk_event_key (GtkWidget *widget, guint keyval, const gchar *
 gchar *e_utf8_from_gtk_string (GtkWidget *widget, const gchar *string);
 gchar *e_utf8_from_gtk_string_sized (GtkWidget *widget, const gchar *string, gint bytes);
 
-gchar * e_utf8_to_gtk_string (GtkWidget *widget, const gchar *string);
-gchar * e_utf8_to_gtk_string_sized (GtkWidget *widget, const gchar *string, gint bytes);
+gchar *e_utf8_to_gtk_string (GtkWidget *widget, const gchar *string);
+gchar *e_utf8_to_gtk_string_sized (GtkWidget *widget, const gchar *string, gint bytes);
 
+gchar *e_utf8_from_locale_string (const gchar *string);
+gchar *e_utf8_from_locale_string_sized (const gchar *string, gint bytes);
+
+gchar *e_utf8_to_locale_string (const gchar *string);
+gchar *e_utf8_to_locale_string_sized (const gchar *string, gint bytes);
 /*
  * These are simple wrappers that save us some typing
  */
@@ -56,6 +73,9 @@ GtkWidget *e_utf8_gtk_menu_item_new_with_label (GtkMenu *menu, const gchar *labe
 void e_utf8_gtk_clist_set_text (GtkCList *clist, gint row, gint col, const gchar *text);
 gint e_utf8_gtk_clist_append (GtkCList *clist, gchar *text[]);
 
+gchar * e_utf8_xml1_decode (const gchar *text);
+gchar * e_utf8_xml1_encode (const gchar *text);
+
 gint g_unichar_to_utf8 (gint c, gchar *outbuf);
 guint32 gdk_keyval_to_unicode (guint keysym);
 
