@@ -23,7 +23,7 @@ typedef struct _ColorCombo {
 } ColorCombo;
 
 typedef struct {
-	GnomeCanvasClass parent_class;
+	GtkComboBoxClass parent_class;
 
 	/* Signals emited by this widget */
 	void (* changed) (ColorCombo *color_combo, GdkColor *color);
