@@ -44,7 +44,7 @@ GtkWidget *pixmap_combo_new           (PixmapComboElement const *elements,
 void       pixmap_combo_select_pixmap (PixmapCombo *combo, int index);
 				  
 typedef struct {
-	GnomeCanvasClass parent_class;
+	GtkComboBoxClass parent_class;
 
 	/* Signals emited by this widget */
 	void (* changed) (PixmapCombo *pixmap_combo, int index);
