@@ -38,7 +38,7 @@
  * 3   Date
  */
 
-#define INITIAL_SPEC "<ETableSpecification>                    	       \
+#define INITIAL_SPEC "<ETableSpecification cursor-mode=\"line\" selection-mode=\"browse\" draw-focus=\"true\">                    	       \
   <ETableColumn model_col=\"0\" _title=\"Subject\"   expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"tree-string\" compare=\"string\"/> \
   <ETableColumn model_col=\"1\" _title=\"Full Name\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
   <ETableColumn model_col=\"2\" _title=\"Email\"     expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
@@ -264,6 +264,7 @@ save_state (GtkButton *button, gpointer data)
 	ETreeModel *e_tree_model = E_TREE_MODEL (data);
 
 	e_tree_model_save_expanded_state (e_tree_model, "expanded_state");
+	e_table_save_state (E_TABLE(e_table), "header_state");
 }
 
 /* We create a window containing our new tree. */
@@ -331,6 +332,7 @@ create_tree (void)
 	 * initial layout.  It does the rest.
 	 */
 	e_table = e_table_new (E_TABLE_MODEL(e_tree_model), NULL, INITIAL_SPEC, NULL);
+	e_table_load_state (E_TABLE(e_table), "header_state");
 
 	if (!e_table) printf ("BAH!");
 
